#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
Configuration module, which reads and parses the configuration file.
"""

import ConfigParser
import sys, traceback
import argparse

# Dictionary with the required present fields in the config file
requiredConfig = {
        'server' : [ 'port' ] ,
        'logging' : [ 'path', 'level', 'stderr', 'datefmt' ],
        'api' : [ 'user', 'pass' ], 
        'postgrest': [ 
            'rundeck_resources_url', 
            'host_aliases_url', 
            'metadata_url', 
            'instance_url', 
            'volume_url', 
            'attribute_url', 
            'functional_alias_url'
            ], 
        }

optionalConfig = {
                  'rundeck' : [ 
                      'timeout',
                      'api_run_job', 
                      'api_job_output', 
                      'api_authorization'
                      ], 
                  'rundeck-jobs' : [
                      'get-snapshots'
                      ]
                  }


def load( config_file = None ):
    """
    Reads configuration file
    """
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument(
                "-c", "--config", default="/etc/apiato/api.cfg", 
                help = "specify the location of the config file"
                )
        args, unk = parser.parse_known_args()

        # Load configuration from file
        config = ConfigParser.ConfigParser()
        config.add_section('tornado')
        config.set('tornado', 'debug', 'false')
        if config_file:
            number_read_files = config.read(config_file)
        else:
            number_read_files = config.read(args.config)

        # check the config file exist and can be read
        if len(number_read_files) != 1:
            print "Configuration file '{0}' cannot be read or does not exist. Stopping.".format(args.config)
            sys.exit(1)
        
        # Force load of all required fields to avoid errors in runtime
        for section, options in requiredConfig.items():
            for option in options:
                try:
                    config.get(section, option)
                except ConfigParser.NoSectionError:
                    print "Section '{0}' not present in config file. Stopping.".format(section)
                    sys.exit(2)
                except ConfigParser.NoOptionError:
                    print "Option '{0}' not present in section {1} in config file. Stopping.".format(option, section)
                    sys.exit(2)
        return config
        
    except IOError as e:
        traceback.print_exc(file=sys.stdout)
        sys.exit(e.code)

# Loads config
config = load()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2017, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.


#Functional alias module

import logging
import json
import requests
import tornado.web
import tornado.escape
from apiato.api.base import *
from apiato.config import config

class Resources(tornado.web.RequestHandler):
    """
    This is the handler of **/auth/resources** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's view that is used for this endpoint is called *api.e_groups* and provides a list of the e_groups owning instances.
    
    The request methods implemented for this endpoint are:

    * :func:`get`

    """

    url = config.get('postgrest', 'egroups_url')

    def get(self, *args):

        """
        The *GET* method returns a list of e_groups owning resources

        :rtype: json -- the response of the request
        :raises: HTTPError - if there is an internal error or if the response is empty
        """

        composed_url = self.url
        try:
            logging.debug('Request body: %s' % (self.request.body))
            body = json.loads(self.request.body)
            username = body.get("username")
            user_egroups = set(body.get("groups"))
        except KeyError, ValueError:
            raise tornado.web.HTTPError(BAD_REQUEST)
        
        resources = {}
        
        # Computing group intersection
        logging.debug('User e_groups %s' % user_egroups)

        # Verifying Admin role
        if config.get('auth', 'admin_group') in list(user_egroups):
            resources["admin"] = True

        logging.debug("Requesting system e_groups" + composed_url)
        response = requests.get(composed_url)
        system_egroups = None
        data = response.json()
        if response.ok and data:
            logging.debug("response: " + json.dumps(data))
            system_egroups = data[0].get("e_groups")
        else:
            logging.error("Error fetching list of system recognized e_groups: " + response.text)
            raise tornado.web.HTTPError(response.status_code)
        
        intersect = user_egroups.intersection(system_egroups)
        if bool(intersect):
            resources["groups"] = list(intersect)
        
        # Fetching list of owned instances
        logging.debug('Username: %s' % (username) )
        logging.debug("Requesting user owned instances: " + config.get('postgrest', 'user_instances_url'))
        rbody = {'owner': username , 'groups': resources.get('groups')}
        response = requests.post(config.get('postgrest','user_instances_url'),
            json=rbody,
            headers={'Prefer': 'return=representation'}
            )
        if response.ok:
            data = response.json()
            logging.debug("User instances response: " + json.dumps(data))
            resources["instances"] = data[0].get("get_user_instances")
        else:
            logging.debug(response)
            logging.info("No instances directly owned by user")
        
        # Fetching list of owned clusters
        logging.debug('Username: %s' % (username) )
        logging.debug("Requesting user owned clusters: " + config.get('postgrest', 'user_clusters_url'))
        rbody = {'owner': username , 'groups': resources.get('groups')}
        response = requests.post(config.get('postgrest','user_clusters_url'),
            json=rbody,
            headers={'Prefer': 'return=representation'}
            )
        if response.ok:
            data = response.json()
            logging.debug("User clusters response: " + json.dumps(data))
            resources["clusters"] = data[0].get("get_user_clusters")
        else:
            logging.debug(response)
            logging.info("No instances directly owned by user")

        if bool(resources):
            self.write(json.dumps(resources))
            self.set_status(OK)
        else:
            raise tornado.web.HTTPError(NOT_FOUND)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import tornado.web
import json
import urllib
import logging
import base64

from tornado.testing import AsyncHTTPTestCase
from tornado.testing import get_unused_port
from timeout_decorator import timeout

from apiato.api.api import *

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class AttributeTest(AsyncHTTPTestCase):
    """Class to test attributes endpoint"""
    authentication = "basic " + base64.b64encode(config.get('api','user') + ":" + config.get('api','pass'))
    
    def get_app(self):
        return tornado.web.Application(handlers, debug=True)
    
    @timeout(5)
    def create_instance(self):
        """Create a new empty instance to run the tests to it"""
        response = self.fetch("/api/v1/instance/testdbat", method='DELETE', headers={'Authorization': self.authentication})
        
        instance = """{
        "owner": "testuser", "category": "TEST", "creation_date":"2016-07-20", 
        "version": "5.6.17", "type_id": 2, "host_id": 1, "name": "testdbat", "state": "MAINTENANCE", "status": "ACTIVE",
        "volumes": [
            {"group": "ownergroup", "file_mode": "0755", "server": "NAS-server", "mount_options": "rw,bg,hard", "volume_type_id": 1,
            "owner": "TSM", "mounting_path": "/MNT/data1"}, 
            {"group": "ownergroup", "file_mode": "0755", "server": "NAS-server", "mount_options": "rw,bg,hard", "volume_type_id": 1,
            "owner": "TSM", "mounting_path": "/MNT/bin"}
        ],
        "attributes": [{
            "port": "5505",
            "notifications": "false"
        }]}"""
        
        # Create the instance
        response = self.fetch("/api/v1/instance/testdbat", method='POST', headers={'Authorization': self.authentication}, body=instance)
        
    @timeout(5)
    def delete_instance(self):
        """Destroy the created test instance"""
        response = self.fetch("/api/v1/instance/testdbat", method='DELETE', headers={'Authorization': self.authentication})
        

    @timeout(5)
    def test_get_attribute(self):
        """Get the port attribute"""
        self.create_instance()
        
        response = self.fetch("/api/v1/instance/testdbat/attribute/port")
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        port = json.loads(response.body).get('value')
        self.assertEquals(port, "5505")
        
        self.delete_instance()
    
    @timeout(5)
    def test_get_attributes(self):
        """Get all the instance attributes"""
        self.create_instance()
        metadata = self.fetch("/api/v1/instance/testdbat/metadata")

        logging.debug("Metadata: %s" % (metadata))

        response = self.fetch("/api/v1/instance/testdbat/attribute")
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        data = json.loads(response.body)
        port = data.get('port')
        notifications = data.get('notifications')
        self.assertEquals(len(data), 2)
        self.assertEquals(port, "5505")
        self.assertEquals(notifications, "false")
        
        self.delete_instance()
        
    @timeout(5)
    def test_get_attribute_no_instance(self):
        """Get the port attribute when instance doesn't exist"""
        response = self.fetch("/api/v1/instance/testdbatnoexist/attribute/port")
        self.assertEquals(response.code, 404)
    
    @timeout(5)
    def test_get_attributes_no_instance(self):
        """Get all the instance attributes when instance doesn't exist"""
        response = self.fetch("/api/v1/instance/testdbatnoexist/attribute")
        self.assertEquals(response.code, 404)
        
    @timeout(5)
    def test_add_attribute(self):
        """Add a new attribute to the instance"""
        self.create_instance()
        
        attribute = """{"test_attribute": "test_value"}"""
        response = self.fetch("/api/v1/instance/testdbat/attribute", method='POST', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 201)
        
        # Get the new attribute
        response = self.fetch("/api/v1/instance/testdbat/attribute/test_attribute")
        self.assertEquals(response.code, 200)
        value = json.loads(response.body).get('value')
        self.assertEquals(value, "test_value")
        
        
        # Get and check all the attributes
        response = self.fetch("/api/v1/instance/testdbat/attribute")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)
        logging.debug("attributes: %s" % (data))
        self.assertEquals(len(data), 3)
        self.assertEquals(data["test_attribute"], "test_value")
        
        self.delete_instance()
        
    @timeout(5)
    def test_add_attributes(self):
        """Add some new attributes to the instance"""
        self.create_instance()
        
        attribute = """{"test_attribute1": "test_value1", "test_attribute2": "test_value2"}"""
        response = self.fetch("/api/v1/instance/testdbat/attribute", method='POST', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 201)
        
        # Get the new attribute
        response = self.fetch("/api/v1/instance/testdbat/attribute/test_attribute1")
        self.assertEquals(response.code, 200)
        value = json.loads(response.body).get('value')
        self.assertEquals(value, "test_value1")
        response = self.fetch("/api/v1/instance/testdbat/attribute/test_attribute2")
        self.assertEquals(response.code, 200)
        value = json.loads(response.body).get('value')
        self.assertEquals(value, "test_value2")
        
        # Get and check all the attributes
        response = self.fetch("/api/v1/instance/testdbat/attribute")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)
        self.assertEquals(len(data), 4)
        self.assertEquals(data["test_attribute1"], "test_value1")
        self.assertEquals(data["test_attribute2"], "test_value2")
        
        self.delete_instance()
        
    @timeout(5)
    def test_add_null_attribute(self):
        """Add a null attribute to the instance"""
        self.create_instance()
        
        attribute = """{"test_attribute": null}"""
        response = self.fetch("/api/v1/instance/testdbat/attribute", method='POST', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 400) # Bad Request
        
        self.delete_instance()
        
    @timeout(5)
    def test_add_existing_attribute(self):
        """Add an existing attribute to the instance"""
        self.create_instance()
        
        attribute = """{"port": "5500"}"""
        response = self.fetch("/api/v1/instance/testdbat/attribute", method='POST', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 409) # Conflict
        
        self.delete_instance()
        
    @timeout(5)
    def test_add_attribute_no_instance(self):
        """Add a new attribute to an instance doesn't exist"""
        attribute = """{"test_attribute": "test_value"}"""
        response = self.fetch("/api/v1/instance/testdbat/attribute", method='POST', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 400)
        
    @timeout(5)
    def test_edit_existing_attribute(self):
        """Edit an existing attribute of the instance"""
        self.create_instance()
        
        attribute = "6611"
        response = self.fetch("/api/v1/instance/testdbat/attribute/port", method='PUT', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 200)
        
        # Get the new value
        response = self.fetch("/api/v1/instance/testdbat/attribute/port")
        self.assertEquals(response.code, 200)
        port = json.loads(response.body).get('value')
        self.assertEquals(port, "6611")
        
        self.delete_instance()
        
    @timeout(5)
    def test_edit_non_existing_attribute(self):
        """Edit a non existing attribute of the instance"""
        self.create_instance()
        
        attribute = "cls01"
        response = self.fetch("/api/v1/instance/testdbat/attribute/cluster", 
                method='PUT', headers={'Authorization': self.authentication}, body = attribute)
        self.assertEquals(response.code, 500)
        
        # Check it was not created
        response = self.fetch("/api/v1/instance/testdbat/attribute/cluster")
        self.assertEquals(response.code, 404)
        
        self.delete_instance()
        
    @timeout(5)
    def test_edit_attribute_no_instance(self):
        """Edit an attribute of a non existing instance"""
        attribute = "6611"
        response = self.fetch("/api/v1/instance/testdbnot/attribute/port", method='PUT', headers={'Authorization': self.authentication}, body=attribute)
        self.assertEquals(response.code, 500)
        
    @timeout(5)
    def test_delete_existing_attribute(self):
        """Delete an existing attribute of the instance"""
        self.create_instance()
        
        response = self.fetch("/api/v1/instance/testdbat/attribute/port", method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)
        
        # Get the new value
        response = self.fetch("/api/v1/instance/testdbat/attribute/port")
        self.assertEquals(response.code, 404)
        
        self.delete_instance()
        
    @timeout(5)
    def test_delete_non_existing_attribute(self):
        """Delete an non existing attribute of the instance"""
        self.create_instance()
        
        response = self.fetch("/api/v1/instance/testdbat/attribute/cluster", method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)
        
        self.delete_instance()
        
    @timeout(5)
    def test_delete_attribute_no_instance(self):
        """Delete an attribute of a non existing instance"""
        response = self.fetch("/api/v1/instance/testdbnot/attribute/port", method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)

    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.


#Functional alias module

import logging
import json
import requests
import tornado.web
import tornado.escape
from apiato.api.base import *
from apiato.config import config

class Host(tornado.web.RequestHandler):
    """
    This is the handler of **/host/names/<name>** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table that is used for this endpoint is called *host* and provides information for the functional alias association with an instance.
    * The columns of this table are like that:

    +----+-----------+--------+
    | id |  name     | memory |
    +====+===========+========+
    | 42 | apiato-db42 |  1024  |
    +----+-----------+--------+

        * The *name* in this example is the hostname of a node
        * The *memory* is the memory of the node expressed as integer in megabytes

    The request methods implemented for this endpoint are:

    * :func:`get`
    * :func:`post`
    * :func:`put`
    * :func:`delete` 

    .. note::

      You need to provide a <*username*> and a <*password*> to use
      :func:`post`, :func:`put`, and :func:`delete` methods or to 
      provide manually the *Authorization* header.

    """

    url = config.get('postgrest', 'host_url')

    def get(self, name, *args):
        """
        The *GET* method returns the host's memory according to the example given above.
        (No any special headers for this request)

        :param name: the hostname which is given in the url
        :type name: str
        :rtype: json -- the response of the request
        :raises: HTTPError - when the requested name does not exist or if there is an internal 
        error or if the response is empty
        """

        logging.debug('Arguments:' + str(self.request.arguments))
        composed_url = self.url + '?name=eq.' + name + '&select=memory'
        logging.info("Requesting " + composed_url)
        response = requests.get(composed_url)
        data = response.json()
        if response.ok and data:
            logging.debug("response: " + json.dumps(data))
            self.write({'response' : data})
        elif response.ok:
            logging.warning("Name provided is not found in the table of the database: " + name)
            raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Error fetching name: " + response.text)
            raise tornado.web.HTTPError(response.status_code)

    @http_basic_auth
    def post(self, name, *args):
        """
        The *POST* method inserts a new *name* and its *memory* size according to the example above. 
        You don't have to specify anything about the *id* since this field should be specified as *serial*.

        .. note::

            This method is not successful:

            * if the *name* already exists
            * if the format of the *request body* is not right
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param name: the new name which is given in the url
        :type name: str
        :raises: HTTPError - when the *url* or the *request body* format or the *headers* are not right orthere is an internal error
        :request body: memory=<memory_int_MB> - the memory (integer in MB) to be inserted for the given *name* which is given in the *body* of the request
        """

        logging.debug('Arguments:' + str(self.request.arguments))
        try:
            memory = int(self.get_argument('memory'))
            logging.debug("memory: %s" %(memory))

            headers = {'Prefer': 'return=representation', 
                        'Content-Type': 'application/json'}
            insert_data = {"name": name,
                           "memory": memory}
            logging.debug("Data to insert: %s" %(insert_data))
            composed_url = self.url + '?name=eq.' + name
            logging.debug('Requesting insertion: ' + composed_url)
            
            response = requests.post(composed_url, json=insert_data, headers=headers)
            if response.ok:
                    logging.info('Data inserted in the table')
                    logging.debug(response.text)
                    self.set_status(CREATED)
            else:
                    logging.error("Duplicate entry or backend problem: " + response.text)
                    self.set_status(response.status_code)
                    raise tornado.web.HTTPError(response.status_code)

        except tornado.web.MissingArgumentError:
            logging.error("Bad argument given in the POST body request")
            logging.error("Try entering 'memory=<memory_int_MB>'")
            logging.error("Or try adding this header: \
                          'Content-Type: application/x-www-form-urlencoded'")
            self.set_status(BAD_REQUEST)
            raise tornado.web.HTTPError(BAD_REQUEST)
        except ValueError:
            logging.error("The value of the argument in the request body \
                        should be an integer")
            logging.error("Try entering 'memory=<memory_int_MB>'")
            self.set_status(BAD_REQUEST)
            raise tornado.web.HTTPError(BAD_REQUEST)

    @http_basic_auth   
    def put(self, name, *args):
        """
        The *PUT* method updates the *memory* size of the given *name* according to the example above.
        You don't have to specify anything about the *id* since this field should be specified as *serial*.


        .. note::
            
            In order to be able to use *PUT* method possibly you have to specify this header:
            'Content-Type: application/x-www-form-urlencoded'.

            This method is not successful:

            * if the *name* does not exist
            * if the format of the *request body* is not right
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param name: the new name which is given in the url
        :type name: str
        :raises: HTTPError - when the *url* or the *request body* format or the *headers* are not right orthere is an internal error
        :request body: memory=<memory_int_MB> - the memory (integer in MB) to be inserted for the given *name* which is given in the *body* of the request
        """

        logging.debug('Arguments:' + str(self.request.arguments))
        try:
            memory = int(self.get_argument('memory'))
            logging.debug("memory: %s" %(memory))
                
            headers = {'Prefer': 'return=representation', 
                    'Content-Type': 'application/json'}
            update_data = {"name": name,
                        "memory": memory}
            logging.debug("Data to insert: %s" %(update_data))
            composed_url = self.url + '?name=eq.' + name
            logging.debug('Requesting insertion: ' + composed_url)
            response = requests.patch(composed_url, json=update_data, headers=headers)
            if response.ok:
                logging.info('Data updated in the table')
                logging.debug(response.text)
                self.set_status(OK)
            else:
                logging.error("Error while updating the new name: " + response.text)
                self.set_status(response.status_code)
                raise tornado.web.HTTPError(response.status_code)

        except tornado.web.MissingArgumentError:
            logging.error("Bad argument given in the POST body request")
            logging.error("Try entering 'memory=<memory_int_MB>'")
            logging.error("Or try adding this header: \
                            'Content-Type: application/x-www-form-urlencoded'")
            self.set_status(BAD_REQUEST)
            raise tornado.web.HTTPError(BAD_REQUEST)

        except ValueError:
            logging.error("The value of the argument in the request body \
                    should be an integer")
            logging.error("Try entering 'memory=<memory_int_MB>'")
            self.set_status(BAD_REQUEST)
            raise tornado.web.HTTPError(BAD_REQUEST)

    @http_basic_auth
    def delete(self, name, *args):
        """
        The *DELETE* method deletes an entry from the table given the *name* in the url.
        You don't have to specify anything about the *id*.


        .. note::
            
            This method is not successful:

            * if the *name* does not exist
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param name: the new name which is given in the url
        :type name: str
        :raises: HTTPError - when there is an internal error or the requested name does not exist
        :request body: memory=<memory_int_MB> - the memory (integer in MB) to be inserted for the given *name* which is given in the *body* of the request
        """
        headers = {'Prefer': 'return=representation',
            'Content-Type': 'application/json'}
        composed_url = self.url + '?name=eq.' + name
        response = requests.delete(composed_url, headers=headers)
        logging.info("Requesting deletion of: " + name)
        if response.ok:
            logging.info("Data deleted")
            logging.debug(response.text)
            self.set_status(NO_CONTENT)
        else:
            logging.error("Error during deletion with code %s: %s" \
                    %(response.status_code, response.text) )
            logging.error("The given name does not exist in the table")
            raise tornado.web.HTTPError(response.status_code)




# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import unittest
from types import *
import json
import logging
import sys
import requests

import base64
import tornado.web

from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from apiato.api.api import handlers
from apiato.config import config
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class HostTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing Host endpoint with nosetest"""
    
    authentication = "basic " + \
                     base64.b64encode(config.get('api', 'user') + \
                     ":" + config.get('api', 'pass'))

    def get_app(self):
        return tornado.web.Application(handlers)

    @timeout(5)
    @patch('apiato.api.host.requests.get')
    @patch('apiato.api.host.json.dumps')
    @patch('apiato.api.host.Host.write')
    def test_get_valid_name(self, mock_write, mock_json, mock_get):
        """ test a successful get method with a valid given name"""
        print "test_get_valid_name"
        status_code_test = 200
        response_output = [{u'memory': 512}]
        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          ok=True,
                                          status_code=status_code_test,
                                          content=response_output)
        #mock_get.json.return_value.content = response_output
        #mock_json.return_value = MagicMock(content=[{"memory": 512}])
        response = self.fetch("/api/v1/host/names/host42")
        self.assertEquals(response.code, status_code_test)
    
    def empty_json(*args,**kwargs):
        class MockResponse:
            
            def __init__(self, ok, status_code, json_data):
                self.status_code_test = status_code
                self.ok = ok
                self.json_data = json_data

            def json(self):
                return self.json_data
        return MockResponse(True,200,[])


    @timeout(5)
    @patch('apiato.api.host.requests.get', side_effect=empty_json)
    def test_get_empty(self, mock_get):
        """test when the response of the request is empty"""
        print "test_get_empty"

        response = self.fetch("/api/v1/host/names/host42")
        self.assertEquals(response.code, 404)
    
    @timeout(5)
    @patch('apiato.api.host.requests.get')
    def test_get_notexist(self, mock_get):
        """test when the given name does not exist"""
        print "test_get_notexist"
        
        status_code_test_error = 502
        response_output = [{u'memory': 512}]
        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test_error,
                                          content=response_output)
        response = self.fetch("/api/v1/host/names/host42")
        self.assertEquals(response.code, status_code_test_error)
    
    @timeout(5)
    @patch('apiato.api.host.requests.post')
    def test_post_valid(self, mock_post):
        """test when the post request is valid"""
        print "test_post_valid"

        status_code_test = 201
        memory_test = '512'
        body_test = 'memory=' + memory_test
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                          ok=True,
                                          status_code=status_code_test)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test)
   
    @timeout(5)
    @patch('apiato.api.host.requests.post')
    def test_post_duplicate(self, mock_post):
        """test when the post request tries to insert a duplicate entry"""
        print "test_post_duplicate"

        status_code_test_error = 409
        memory_test = '512'
        body_test = 'memory=' + memory_test
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test_error)

    @timeout(5)
    @patch('apiato.api.host.requests.post')
    def test_post_wrongargument(self, mock_post):
        """test when the argument in the body of post request is wrong"""
        print "test_post_duplicate"

        status_code_test_error = 400
        memory_test = '512'
        body_test = 'something=' + memory_test
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test_error)
    
    @timeout(5)
    @patch('apiato.api.host.requests.post')
    def test_post_badargument(self, mock_post):
        """test when the value of the argument of post request is string"""
        print "test_post_badargument"

        status_code_test_error = 400
        memory_test = 'forty-two'
        body_test = 'memory=' + memory_test
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test_error)

    @timeout(5)
    @patch('apiato.api.host.requests.patch')
    def test_put_valid(self, mock_patch):
        """test when the put request is valid"""
        print "test_put_valid"

        status_code_test = 200
        memory_test = '42'
        body_test = 'memory=' + memory_test
        mock_patch.return_value = MagicMock(spec=requests.models.Response,
                                          ok=True,
                                          status_code=status_code_test)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="PUT", 
                              headers={'Authorization': self.authentication,
                                       'Content-Type': 'application/x-www-form-urlencoded'},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test)

    @timeout(5)
    @patch('apiato.api.host.requests.patch')
    def test_put_notexist(self, mock_patch):
        """test when the name to update with put request does not exist"""
        print "test_put_notexist"

        status_code_test_error = 404
        memory_test = '512'
        body_test = 'memory=' + memory_test
        mock_patch.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="PUT", 
                              headers={'Authorization': self.authentication,
                                       'Content-Type': 'application/x-www-form-urlencoded'},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test_error)


    @timeout(5)
    @patch('apiato.api.host.requests.patch')
    def test_put_wrongargument(self, mock_patch):
        """test when the argument in the body of put request is wrong"""
        print "test_put_wrongargument"

        status_code_test_error = 400
        memory_test = '42'
        body_test = 'something=' + memory_test
        mock_patch.return_value = MagicMock(spec=requests.models.Response,
                                          status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="PUT", 
                              headers={'Authorization': self.authentication,
                                       'Content-Type': 'application/x-www-form-urlencoded'},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test_error)
    
    @timeout(5)
    @patch('apiato.api.host.requests.patch')
    def test_put_badargument(self, mock_patch):
        """test when the value of the argument of put request is string"""
        print "test_put_badargument"

        status_code_test_error = 400
        memory_test = 'forty-two'
        body_test = 'memory=' + memory_test
        mock_patch.return_value = MagicMock(spec=requests.models.Response,
                                          status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="PUT", 
                              headers={'Authorization': self.authentication,
                                       'Content-Type': 'application/x-www-form-urlencoded'},
                              body=body_test)  
        self.assertEquals(response.code, status_code_test_error)
    
    @timeout(5)
    @patch('apiato.api.host.requests.delete')
    def test_delete_valid(self, mock_delete):
        """test when the delete request is valid"""
        print "test_delete_valid"

        status_code_test = 204
        memory_test = '512'
        body_test = 'memory=' + memory_test
        mock_delete.return_value = MagicMock(spec=requests.models.Response,
                                             ok=True,
                                             status_code=status_code_test)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="DELETE", 
                              headers={'Authorization': self.authentication})  
        self.assertEquals(response.code, status_code_test)

    @timeout(5)
    @patch('apiato.api.host.requests.delete')
    def test_delete_notexist(self, mock_delete):
        """test when the name to delete with delete request does not exist"""
        print "test_delete_notexist"

        status_code_test_error = 404
        memory_test = '512'
        body_test = 'memory=' + memory_test
        mock_delete.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test_error)

        response = self.fetch("/api/v1/host/names/host42", 
                              method="DELETE", 
                              headers={'Authorization': self.authentication})
        self.assertEquals(response.code, status_code_test_error)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
cluster module, which includes all the classes related with cluster endpoints.
"""

import tornado.web
import logging
import requests
import json

from apiato.api.base import *
from apiato.config import config

class Cluster(tornado.web.RequestHandler):
    """
    This is the handler of **/cluster/<name>** endpoint.
    This endpoint takes 1 arguments:
    * *<name>* - the name of a *cluster*
    Things that are given for the development of this endpoint:
    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table/view that is used for this endpoint is called *cluster* and provides metadata about a cluster and its instances.
    * Here is an example of this table:
    --ToDO
    The request method implemented for this endpoint is just the :func:`get`.
    """
    def get(self, name):
        """Returns the cluster information
        he *GET* method returns a *cluster* given a *name*.
        (No any special headers for this request)
        :param name: the database name which is given in the url
        :type name: str
        :rtype: json - the response of the request
        :raises: HTTPError - when the given cluster name does not exist or in case of an internal error
        """
        response = requests.get(config.get('postgrest', 'cluster_url') + "?name=eq." + name)
        if response.ok:
            data = response.json()
            if data:
                self.write({'response' : data})
                self.set_status(OK)
            else:
                logging.error("Cluster metadata not found: " + name)
                raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Entity metadata not found: " + name)
            raise tornado.web.HTTPError(NOT_FOUND)



    @http_basic_auth
    def post(self, id):
        """
        The *POST* method inserts a new cluster into the database with all the
        information that is needed for the creation of it.
        In the request body we specify all the information of the *cluster*
        table along with the *attribute* table. We extract and
        separate the information of each table.
        .. note::
            * It's possible to insert more than one *attribute* in one cluster.
            * The cluster names have to be unique
            * Also, the creation is not successful
                * if the client is not authorized or
                * if there is any internal error
                * if the format of the request body is not right or if there is no *database name* field
        :param id: the new cluster name which is given in the url or any other string
        :type id: str
        :raises: HTTPError - in case of an internal error
        :request body:  json
                       - for *cluster*: json
                       - for *cluster attribute*: json
        """
        logging.debug(self.request.body)
        cluster = {'in_json': json.loads(self.request.body)}

        # Insert the instance in database using PostREST
        response = requests.post(config.get('postgrest', 'insert_cluster_url'), json=cluster, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Created cluster " + cluster["in_json"]["name"])
            logging.debug(response.text)
            self.set_status(CREATED)
        else:
            logging.error("Error creating the cluster: " + response.text)
            raise tornado.web.HTTPError(response.status_code)




    @http_basic_auth
    def put(self, id):
        """
        The *PUT* method updates a cluster with all the information that is needed.
        The procedure of this method is the following:
        :param name: the cluster name which is given in the url
        :type name: str
        :raises: HTTPError - when the *request body* format is not right or in case of internall error
        """
        logging.debug(self.request.body)
        cluster = {'cid': id, 'in_json': json.loads(self.request.body)}
        response = requests.post(config.get('postgrest', 'update_cluster_url'), json=cluster, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Update Cluster: " + id)
            logging.debug(response.text)
            self.set_status(OK)
        else:
            logging.error("Error updating the cluster: " + response.text)
            raise tornado.web.HTTPError(response.status_code)


    @http_basic_auth
    def delete(self, id):
        """
        The *DELETE* method deletes a cluster by *name*.
        In order to delete a cluster we have to delete all the related information of the specified database name in *cluster*, *attribute* and *instance* tables.
        :param id: the database name which is given in the url
        :type id: str
        :raises: HTTPError - when the given database name cannot be found
        """
        logging.debug(self.request.body)
        cluster = {'cid': id}

        # Insert the instance in database using PostREST
        response = requests.post(config.get('postgrest', 'delete_cluster_url'), json=cluster, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Delete cluster: " + id)
            logging.debug(response.text)
            self.set_status(NO_CONTENT)
        else:
            logging.error("Error delete the cluster: " + response.text)
            raise tornado.web.HTTPError(response.status_code)


class Cluster_filter(tornado.web.RequestHandler):
    """
    This is the handler of **/api/v1/cluster** endpoint.

    The request methods implemented for this endpoint are:

    * :func:`get`

    """

    get_clusters_url = config.get('postgrest', 'get_clusters_url')

    def get(self, *args):
        """
        The *GET* method returns a list of e_groups owning resources

        :rtype: json -- the response of the request
        :raises: HTTPError - if there is an internal error or if the response is empty
        """
        auth_header = self.request.headers.get('auth')
        logging.debug("Auth header : %s" % (auth_header))
        if auth_header is None:
            raise tornado.web.HTTPError(BAD_REQUEST, "No 'auth' header found.")

        try:
            auth = json.loads(auth_header)
        except:
            raise tornado.web.HTTPError(BAD_REQUEST, "Error parsing JSON 'auth' header.")

        logging.debug("RPC Url : %s" % (self.get_clusters_url))

        response = requests.post(self.get_clusters_url, json=auth,
                headers={'Prefer': 'return=representation'})

        if response.ok:
            self.write(response.text)
            self.set_status(OK)
        else:
            logging.error("Response: %s" % (response.text))
            raise tornado.web.HTTPError(response.status_code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
DB On Demand metadata REST API server
"""




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
Metadata module, which includes all the classes related with metadata endpoints.
"""

import tornado.web
import logging
import requests
import json

from apiato.api.base import *
from apiato.config import config

class Metadata(tornado.web.RequestHandler):
    """
    This is the handler of **/metadata/<class>/<name>** endpoint.

    This endpoint takes 2 arguments:

    * *<class>* - "*host*" or "*instance*" 
    * *<name>* - the name of a *host* or of a *database* 

    Things that are given for the development of this endpoint:
    
    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table/view that is used for this endpoint is called *metadata* and provides information about the metadata of a database instance(s).
    * Here is an example of this table:

    +--+--------+---------+-------------+---------------------------------------+
    |id|username| db_name |    hosts    |               attributes              |
    +==+========+=========+=============+=======================================+
    |42|  apiato  |apiato-db42|{apiato-host42}|{"port": "5432", "shared_buffers": "1"}|
    +--+--------+---------+-------------+---------------------------------------+

    The request method implemented for this endpoint is just the :func:`get`.

    """
    def get(self, **args):
        """Returns the metadata of a host or an instance
        The *GET* method returns the instance(s)' metadata given the *host* or the *database name*. 
        (No any special headers for this request)

        :param class: "host" or "instance"
        :type class: str
        :param name: the host or database name which is given in the url
        :type name: str
        :rtype: json - the response of the request 

                * in case of "*host*" it returns all the instances' metadata that are hosted in the specified host
                * in casse of "*instance*" it returns the metadata of just the given database

        :raises: HTTPError - when the <class> argument is not valid ("host" or "instance") or the given host or database name does not exist or in case of an internal error

        """
        name = args.get('name')
        etype = args.get('class')
        if name:
            if etype == u'instance':
                composed_url = config.get('postgrest', 'metadata_url') + '?db_name=eq.' + name
            elif etype == u'host':
                composed_url = config.get('postgrest', 'metadata_url') + '?hosts=@>.{' + name + '}'
            else:
                logging.error("Unsupported endpoint")
                raise tornado.web.HTTPError(BAD_REQUEST)
            logging.info('Requesting ' + composed_url)
            response = requests.get(composed_url, verify=False)
            data = response.json()
            if response.ok and data:
                logging.debug("response: " + json.dumps(data))
                self.write({'response' : data})
                self.set_status(OK)
            elif response.ok:
                logging.warning("Instance metadata not found: " + name)
                raise tornado.web.HTTPError(NOT_FOUND)
            else:
                logging.error("Error fetching instance metadata: " + response.text)
                raise tornado.web.HTTPError(response.status_code)
        else:
            logging.error("Unsupported endpoint")
            raise tornado.web.HTTPError(BAD_REQUEST)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import tornado.web
import json

from tornado.testing import AsyncHTTPTestCase
from tornado.testing import get_unused_port
from timeout_decorator import timeout

from apiato.api.api import *

class MetadataTest(AsyncHTTPTestCase):
    def get_app(self):
        return tornado.web.Application(handlers)

    @timeout(5)
    def test_single_instance_by_name(self):
        response = self.fetch("/api/v1/instance/apiato01/metadata")
        data = json.loads(response.body)["response"]
        self.assertEquals(response.code, 200)
        self.assertEquals(len(data), 1)
        self.assertEquals(data[0]["db_name"], "apiato01")
        self.assertEquals(len(data[0]["hosts"]), 1)
        self.assertTrue(data[0]["volumes"] != None)
    
    @timeout(5)
    def test_no_instance_by_name(self):
        response = self.fetch("/api/v1/instance/invalid/metadata")
        self.assertEquals(response.code, 404)
    
    @timeout(5)
    def test_single_instance_by_host(self):
        response = self.fetch("/api/v1/host/host03/metadata")
        data = json.loads(response.body)["response"]
        self.assertEquals(response.code, 200)
        self.assertEquals(len(data), 1)
        self.assertTrue(data[0]["volumes"] != None)
        self.assertEquals(data[0]["hosts"][0], "host03")
    

    
    @timeout(5)
    def test_no_instance_by_host(self):
        response = self.fetch("/api/v1/host/invalid/metadata")
        self.assertEquals(response.code, 404)
        
    @timeout(5)
    def test_invalid_class(self):
        response = self.fetch("/api/v1/invalid/invalid/metadata")
        self.assertEquals(response.code, 400)
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
Rundeck module, which includes the rundeck classes and endpoints.
"""

import tornado.web
import logging
import json
import requests
import time

from apiato.api.base import *
from apiato.config import config

class RundeckResources(tornado.web.RequestHandler):
    """The class of /rundeck/resources.xml"""
    def get(self):
        """Returns a valid resources.xml file to import target entities in 
            Rundeck"""
        response = requests.get(config.get('postgrest', 'rundeck_resources_url'))
        if response.ok:
            data = json.loads(response.text)
            d = {}
            for entry in data:
                d[entry[u'name']] = entry
            self.set_header('Content-Type', 'text/xml')
            # Page Header
            logging.debug('<?xml version="1.0" encoding="UTF-8"?>')
            self.write('<?xml version="1.0" encoding="UTF-8"?>')
            logging.debug('<project>')
            self.write('<project>')
            for instance in sorted(d.keys()):
                body = d[instance]
                text = ('<node name="%s" description="" hostname="%s" username="%s" type="%s" subcategory="%s" port="%s" tags="%s"/>' % 
                        ( instance, # Name
                          body.get(u'hostname'),
                          body.get(u'username'),
                          body.get(u'category'), 
                          body.get(u'db_type'), 
                          body.get(u'port'), 
                          body.get(u'tags')
                          ))
                logging.debug(text)
                self.write(text)
                self.set_status(OK)
            logging.debug('</project>')
            self.write('</project>')
        else: 
            logging.error("Error fetching Rundeck resources.xml")
            raise tornado.web.HTTPError(NOT_FOUND)
            
class RundeckJobs(tornado.web.RequestHandler):
    """
    This is the handler of **/rundeck/jobs/<job>/<node>** endpoint.

    The class manages the endpoints used to execute and visualize jobs execution in Rundeck.

    .. note::

        You need to be authenticated in order to execute a job.
    """
    @http_basic_auth
    def get(self, **args):
        """
        The *GET* method returns the output of a job execution"""
        job = args.get('job')
        if (job is None):
            raise tornado.web.HTTPError(BAD_REQUEST, "Parameter 'job' not specified")

        if (not isinstance(job, (int, long))):
            raise tornado.web.HTTPError(BAD_REQUEST, "Parameter 'job' is not a number")

        response = self.__get_output__(job)
        if response.ok:
            logging.debug("response: " + response.text)
            try:
                self.write({'response' : json.loads(response.text)})
                self.set_status(OK)
            except:
                raise tornado.web.HTTPError(NOT_ACCEPTABLE, "Error parsing Rundeck response")
        else:
            logging.error("Error reading job from Rundeck: " + response.text)
            raise tornado.web.HTTPError(response.status_code)

    @http_basic_auth
    def post(self, **args):
        """
        The *POST* method executes a new Rundeck job and returns the output.
        
        The job and its hash has to be defined in the *api.cfg* configuration file in order to a specific job to be able to be executed.
        
        :param job: the name of the job to be executed which is listed in the configuration file
        :type job: str
        :param node: the name of the node you want the job to be executed
        :type node: str
        :raises: HTTPError - if the job didn't succeed or if the timeout has exceeded or in case of an internal error

        When a job is executed the request call hangs and waits for a response for a maximum time of 10 seconds. The api constantly calls rundeck's api to check if the job has finished. When it finishes it prints out the response or raises an error if it didn't succeed.
        """
        job = args.get('job')
        node = args.get('node')
        if (job is None):
            raise tornado.web.HTTPError(BAD_REQUEST, "Parameter 'job' not specified")

        if (node is None):
            raise tornado.web.HTTPError(BAD_REQUEST, "Parameter 'node' not specified")

        try:
            jobid = config.get('rundeck-jobs', job)
            logging.debug("Found 'jobid' for " + job + " = " + jobid)
        except:
            raise tornado.web.HTTPError(BAD_REQUEST, "Job '{0}' is not valid".format(job))

        response_run = self.__run_job__(jobid, node)
        if response_run.ok:
            try:
                data = json.loads(response_run.text)
            except:
                logging.debug("Reponse: " + response_run.text)
                raise tornado.web.HTTPError(NOT_ACCEPTABLE, "Error parsing Rundeck response")

            exid = str(data["id"])
            timeout = int(config.get('rundeck', 'timeout')) * 2
            while timeout > 0:
                response_output = self.__get_output__(exid)
                if response_output.ok:
                    try:
                        output = json.loads(response_output.text)
                    except:
                        continue
                    if output["execCompleted"]:
                        if output["execState"] == "succeeded":
                            logging.debug("response: " + response_output.text)
                            self.set_status(OK)
                            self.finish({'response' : output})
                            return
                        else:
                            logging.warning("The job completed with errors: " + exid)
                            raise tornado.web.HTTPError(BAD_GATEWAY)
                    else:
                        timeout -= 1
                        time.sleep(0.500)
                else:
                    logging.error("Error reading the job from Rundeck: " + response_output.text)
                    raise tornado.web.HTTPError(response_output.status_code)
            if timeout <= 0:
                logging.error("Rundeck job timed out: " + job)
                raise tornado.web.HTTPError(GATEWAY_TIMEOUT)
        else:
            logging.error("Error running the job: " + response_run.text)
            raise tornado.web.HTTPError(response_run.status_code)
        
    def __get_output__(self, execution):
        """Returns the output of a job execution"""
        api_job_output = config.get('rundeck', 'api_job_output').format(execution)
        logging.debug("Sending request: " + api_job_output)
        return requests.get(api_job_output, headers={'Authorization': config.get('rundeck', 'api_authorization')}, verify=False)
        
    def __run_job__(self, jobid, node):
        """Executes a new Rundeck job and returns the output"""
        run_job_url = config.get('rundeck', 'api_run_job').format(jobid)
        headers = {'Authorization': config.get('rundeck', 'api_authorization')}
        data = {'filter':'name: ' + node}
        logging.debug("Sending request: " + run_job_url + " with headers: " + str(headers) + " and data: " + str(data))
        return requests.post(run_job_url, headers=headers, verify=False, data=data)




# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import unittest
from types import *
import json
import logging
import sys
import requests

import base64
import tornado.web

from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from apiato.api.api import handlers
from apiato.config import config
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class JobTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing Job endpoint with nosetest"""
    
    authentication = "basic " + \
                     base64.b64encode(config.get('api', 'user') + \
                     ":" + config.get('api', 'pass'))
    
    def get_app(self):
        return tornado.web.Application(handlers)
        
    @timeout(5)
    def test_instance_list(self):
        """Get the list of instances an user can access (1)"""
        auth_header = '{"owner": "user01", "groups": ["testgroupB"], "admin": false}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 3)
        
    @timeout(5)
    def test_list_instances_user1(self):
        """Get the list of instances an user can access (2)"""
        auth_header = '{"owner": "user01", "groups": [], "admin": false}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 2)
        
    @timeout(5)
    def test_list_instances_user2(self):
        """Get the list of instances an user can access (3)"""
        auth_header = '{"owner": "user01", "groups": ["no_exists"], "admin": false}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 2)
        
    @timeout(5)
    def test_list_instances_user3(self):
        """Get the list of instances an user can access (3)"""
        auth_header = '{"owner": "user01", "groups": ["no_exists"], "admin": false}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 2)
        
    @timeout(5)
    def test_list_instances_user4(self):
        """Get the list of instances an user can access (4)"""
        auth_header = '{"owner": "user01", "groups": [], "admin": false}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 2)
        
    @timeout(5)
    def test_list_instances_user5(self):
        """Get the list of instances an user can access (5)"""
        auth_header = '{"owner": "no_exists", "groups": ["testgroupA"], "admin": false}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 2)
        
    @timeout(5)
    def test_list_instances_admin1(self):
        """Get the list of instances an admin can access (1)"""
        auth_header = '{"owner": "user01", "groups": ["testgroupB"], "admin": true}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 8)
        
    @timeout(5)
    def test_list_instances_admin2(self):
        """Get the list of instances an admin can access (2)"""
        auth_header = '{"owner": "user01", "groups": [], "admin": true}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 8)
        
    @timeout(5)
    def test_list_instances_admin3(self):
        """Get the list of instances an admin can access (3)"""
        auth_header = '{"owner": "no_exists", "groups": [], "admin": true}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body)
        self.assertEquals(len(response), 8)
        
    @timeout(5)
    def test_list_instances_only_owner(self):
        """Get the list of instances if only owner param was specified"""
        auth_header = '{"owner": "user01"}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 400)

    @timeout(5)
    def test_list_instances_only_egroup(self):
        """Get the list of instances if only egroup param was specified"""
        auth_header = '{"egroup": ["testgroupB"]}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 400)
        
    @timeout(5)
    def test_list_instances_only_admin(self):
        """Get the list of instances if only egroup param was specified"""
        auth_header = '{"admin": true}'
        
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 400)

    @timeout(5)
    def test_instance_no_auth_header(self):
        """Get the list of instances sending no auth header"""
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication })
        self.assertEquals(response.code, 400)

    @timeout(5)
    def test_instance_invalid_auth_header(self):
        """Get the list of instances sending invalid auth header"""
        auth_header = '{"owner": "no_exists", "groups": [], "admin": true'  # Missing closing bracket
        response = self.fetch("/api/v1/instance", headers={'Authorization': self.authentication, 'Auth':auth_header })
        self.assertEquals(response.code, 400)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
Main module to initialize the Tornado Server and endpoints
"""

import ConfigParser
import sys, traceback, re
import tornado.web
#import logging

from tornado.options import parse_command_line, options, define
from tornado.log import LogFormatter, logging
from tornado.httpserver import HTTPServer
from tornado.ioloop import IOLoop

from apiato.api.base import DocHandler
from apiato.api.rundeck import RundeckResources, RundeckJobs
from apiato.api.metadata import Metadata
from apiato.api.functionalalias import FunctionalAlias
from apiato.api.hostaliases import HostAliases
from apiato.api.host import Host
from apiato.api.instance import Instance, Instance_filter
from apiato.api.attribute import Attribute
from apiato.api.fim import Fim
from apiato.config import config, optionalConfig
from apiato.api.cluster import Cluster, Cluster_filter
from apiato.api.volume import Volume
from apiato.api.auth import Resources
from apiato.api.job import Job, Job_filter

# This list is a global object because in needs to be accessed
# from the test suites
handlers = [
    (r"/", DocHandler),
    (r"/api/v1/auth/resources", Resources),
    (r"/api/v1/instance/([^/]+)", Instance),
    (r"/api/v1/instance", Instance_filter),
    (r"/api/v1/cluster/([^/]+)", Cluster),
    (r"/api/v1/cluster", Cluster_filter),
    (r"/api/v1/volume/([^/]+)", Volume),
    (r"/api/v1/host/aliases/([^/]+)", HostAliases),
    (r"/api/v1/host/names/([^/]+)", Host),
    (r"/api/v1/instance/alias/?(?P<db_name>[^\/]+)?", FunctionalAlias),
    (r"/api/v1/(?P<class>[^\/]+)/(?P<name>[^\/]+)/metadata", Metadata),
    (r"/api/v1/rundeck/resources.xml", RundeckResources),
    (r"/api/v1/rundeck/job/(?P<job>[^\/]+)/?(?P<node>[^\/]+)?", RundeckJobs),
    (r"/api/v1/(?P<class>[^\/]+)/(?P<entity>[^\/]+)/attribute/?(?P<attribute_name>[^\/]+)?", Attribute),
    (r"/api/v1/instance/(?P<db_name>[^\/]+)/job/?(?P<id>[^\/]+)?", Job),
    (r"/api/v1/job", Job_filter),

    # Deprecated, will be deleted in following versions
    (r"/api/v1/metadata/(?P<class>[^\/]+)/?(?P<name>[^\/]+)?", Metadata),  
    (r"/api/v1/fim/([^/]+)", Fim),
    ]

class Application():
    """
    This is the main entrypoint of the apiato-api where the main parameters are
    specified in order to start the server.  
    """

    def __handler_filter(self, handlers, config, optionalConfig):
        """
        Remove handlers which are not to be used when optional functions are
        not configured 
        """
        # Create a copy of the handler list, which we will modify
        res = list(handlers) 
        logging.debug('Active endpoints: %s' % res)
        for section in optionalConfig.keys():
            logging.info('Checking optional configuration section [%s]' % (section))
            try:
                config.items(section)
            except ConfigParser.NoSectionError:
                logging.info('Section not found. Removing related endpoints')
                # Remove handlers whose url matches the missing optional 
                # section names
                for handler in handlers:
                    url, handler_class = handler
                    if re.search(section, url):
                        logging.warning('Removing endpoint [%s]' % url)
                        res.remove(handler)
        logging.info('Active endpoints: %s' % res)
        return res

    def __init__(self):
        """
        Class constructor. Sets up logging, active handlers and application server
        """
        # Set up log file, level and formatting
        options.log_file_prefix = config.get('logging', 'path')
        options.logging = config.get('logging', 'level')
        options.log_to_stderr = config.getboolean('logging', 'stderr')
        
        # Port and arguments
        port = config.get('server', 'port')
        define('port', default=port, help='Port to be used')
        parse_command_line([])
       
        # Override default logging format and date format
        log_format = config.get('logging', 'fmt', raw = True)
        date_format = config.get('logging', 'datefmt', raw = True)
        if date_format:
            formatter = tornado.log.LogFormatter(fmt = log_format, datefmt = date_format)
            for logger in logging.getLogger('').handlers:
                logging.info('Overriding log format for %s' % (logger))
                logger.setFormatter(formatter)

        # Defining handlers
        # Removing optional handlers from handler list 
        filtered_handlers = self.__handler_filter(handlers, config, optionalConfig)
        logging.info("Defining application (url, handler) pairs")
        application = tornado.web.Application(filtered_handlers, 
                            debug = config.getboolean('tornado', 'debug'))
        
        # Configuring server and SSL
        logging.info("Configuring HTTP server")
        if (config.has_section('ssl')):
            http_server = HTTPServer(application,
                    ssl_options = {
                        "certfile" : config.get('ssl', 'hostcert') ,
                        "keyfile" : config.get('ssl', 'hostkey'),
                        })
        else:
            http_server = HTTPServer(application)
            logging.info("Host certificate undefined, SSL is DISABLED")
            
        # Listening port
        http_server.listen(options.port)
        
        # Starting
        logging.info("Starting application on port: " + str(port))
        tornado.ioloop.IOLoop.instance().start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

#Host Aliases module

import tornado.web
import logging
import requests

from apiato.api.base import *
from apiato.config import config

class HostAliases(tornado.web.RequestHandler):
    """
    This the handler of **/host/aliases/<host>** endpoint.
    
    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table/view that is used for this endpoint is called *host_aliases* and provides information for the instance aliases association with a host.
    * The columns of this table are like that:

    +-------------+----------------------------------------------------------+
    |    host     |                         aliases                          |
    +=============+==========================================================+
    | dbod-host42 | dbod-alias42-user1.cern.ch, dbod-alias42-user2.cern.ch   |
    +-------------+----------------------------------------------------------+

        * The *host* is the hostname of the machine that the database instances has been created
        * The *aliases* are the aliases of the databases that exist in this host/machine 

    The request method implemented for this endpoint is just the :func:`get`.

    """
    def get(self, host):

        """ 
        The *GET* method returns the list of ip-aliases registered in a host.
        (No any special headers for this request)

        :param host: the host name which is given in the url
        :type host: str
        :rtype: json -- the response of the request
        :raises: HTTPError - when the requested host or the requested url does not exist

        """

        composed_url = config.get('postgrest', 'host_aliases_url') + '?host=eq.' + host
        logging.info('Requesting ' + composed_url )
        response = requests.get(composed_url)
        data = response.json()
        if response.ok and data:
            logging.debug("response: " + response.text)
            self.write({'response' : data})
            self.status(OK)
        elif response.ok:
            logging.warning("Host aliases not found: " + host)
            raise tornado.web.HTTPError(NOT_FOUND)
        else: 
            logging.error("Error fetching aliases: " + response.text)
            raise tornado.web.HTTPError(NOT_FOUND)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.


#Functional alias module

import logging
import json
from sys import exc_info
import requests
import tornado.web
import tornado.escape
from apiato.api.base import *
from apiato.config import config

class FunctionalAlias(tornado.web.RequestHandler):

    """
    This is the handler of **/instance/alias/<database name>** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table that is used for this endpoint is called *functional_aliases* and provides information for the functional alias association with an instance.
    * The columns of this table are like that:

    +------------+------------+----------------------+
    |  dns_name  |  db_name   |         alias        |
    +============+============+======================+
    | dbod-dns42 | dbod-db42  | dbod-alias42.cern.ch |
    +------------+------------+----------------------+

        * The *dns_name* is used internally. They point to a list of virtual IP addresses and each bound to a host
        * The *db_name* is the name of the database instance
        * The *alias* is the alias given in order the database to be accessed with that name

    * There is a pool of *dns_names* in this table and the other 2 columns are *NULL* in the begining

    The request methods implemented for this endpoint are:

    * :func:`get`
    * :func:`post`
    * :func:`delete` 

    .. note::

      You need to provide a <*username*> and a <*password*> to use
      :func:`post` and :func:`delete` methods or to provide manually the *Authorization* header.

    """

    url = config.get('postgrest', 'functional_alias_url')

    def get(self, db_name, *args):
        """
        The *GET* method returns the database name's *alias* and *dns name*.
        (No any special headers for this request)

        :param db_name: the database name which is given in the url
        :type db_name: str
        :rtype: json -- the response of the request
        :raises: HTTPError - when the requested database name does not exist or if there is an internal error 
        
        """
        logging.debug('Arguments:' + str(self.request.arguments))
        composed_url = self.url + '?db_name=eq.' + db_name + '&select=dns_name,alias'
        logging.info('Requesting ' + composed_url)
        response = requests.get(composed_url)
        data = response.json()
        if response.ok and data:
            logging.debug("response: " + json.dumps(data))
            self.write({'response' : data})
            self.set_status(OK)
        elif response.ok:
            logging.warning("Functional alias not found for instance: " + db_name)
            raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Error fetching functional alias: " + response.text)
            raise tornado.web.HTTPError(response.status_code)

    @http_basic_auth
    def post(self, db_name, *args):
        """
        The *POST* method inserts a new *database name* and its *alias* into the database. It
        adds the functional alias association for an instance.

        The *dns name* is chosen automatically from a pool; so, in the background this method 
        actually updates the *database name* and *alias* fields, which were *NULL* in the 
        begining.

        .. note::

            This method is not successful:

            * if the *database name* already exists
            * if there are no any *dns names* available
            * if the format of the *request body* is not right
            * if headers have to be specified
            * if the client does not have the right authorization header 
           
        :param db_name: the new database name which is given in the url
        :type db_name: str
        :raises: HTTPError - when the *url* or the *request body* format or the *headers* are not right
        :request body: alias=<alias> - the alias to be inserted for the given *database name* which is given in the *body* of the request

        """

        logging.debug('Arguments:' + str(self.request.arguments))
        try:
            alias = self.get_argument('alias')
            logging.debug("alias: %s" % (alias))
            
            dns_name = self._next_dnsname()

            if dns_name:
                logging.debug("dns_name picked: " + str(dns_name))
                headers = {'Prefer': 'return=representation'}
                insert_data = {"db_name": db_name, 
                               "alias": alias}
                logging.debug("Data to insert: " + str(insert_data))

                composed_url = self.url + '?dns_name=eq.' + dns_name
                logging.debug('Requesting insertion: ' + composed_url)
                
                response = requests.patch(composed_url, json=insert_data, headers=headers)
            
                if response.ok:
                    logging.info('Data inserted in the functional_aliases table')
                    logging.debug(response.text)
                    self.set_status(CREATED)
                else:
                    logging.error("Error inserting the functional alias: " + response.text)
                    self.set_status(response.status_code)
                        
            else:
                logging.error("No dns_name available in the functional_aliases table")
                self.set_status(SERVICE_UNAVAILABLE)
        except:
            logging.error("Argument not recognized or not defined.")
            logging.error("Try adding header 'Content-Type:application/x-www-form-urlencoded'")
            logging.error("The right format should be: alias=<alias>")
            raise tornado.web.HTTPError(BAD_REQUEST)


    @http_basic_auth
    def delete(self, db_name, *args):
        """
        The *DELETE* method deletes or else asssigns to *NULL* the *database name* and 
        *alias* fields. It removes the functional alias association for an instance.

        .. note::
            
            * If the *database name* doesn't exist it doesn't do anything
            * You have to be authorized to use this method
        
        :param db_name: the new database name which is given in the url
        :type db_name: str
        :raises: HTTPError - when the deletion is not successful

        """

        logging.debug('Arguments:' + str(self.request.arguments))

        dns_name = self._get_dns(db_name)
        logging.debug(dns_name)
        if dns_name:
            headers = {'Prefer': 'return=representation', 'Content-Type': 'application/json'}
            composed_url = self.url + '?dns_name=eq.' + dns_name
            logging.debug('Requesting deletion: ' + composed_url)
            delete_data = '{"db_name": null, "alias": null}'
            logging.debug("dns_name to be remained: " + dns_name)
            response = requests.patch(composed_url, json=json.loads(delete_data), headers=headers)

            if response.ok:
                logging.info("Delete success of: " + dns_name)
                logging.debug(response.text)
                self.set_status(NO_CONTENT)
            else:
                logging.error("Unsuccessful deletion")
                raise tornado.web.HTTPError(response.status_code)

        else:
            logging.info("db_name not found. Nothing to do")

    def _next_dnsname(self):
        """
        This is a private function which is used by :func:`post` method.
        Returns the next dnsname which can be used for a newly created instance.  
        If there is no available *dns name* in the pool or if there is any internal error it returns *None*.

        :rtype: str or None

        """
        #LIMIT is not working in postgrest but it uses some headers for that as well
        headers = {'Range-Unit': 'items', 'Range': '0-0'}
        # select the next available dns_name with db_name and alias assigned to NULL
        query_select = '?select=dns_name&order=dns_name.asc&'
        query_filter = 'db_name=is.null&alias=is.null&dns_name=isnot.null'
        composed_url = self.url + query_select + query_filter
        try:
            response_dns = requests.get(composed_url, headers=headers)
            if response_dns.ok:
                response_dns_dict = json.loads(response_dns.text)[0]
                return response_dns_dict['dns_name']
            else:
                return None
        except:
            error_msg = exc_info()[0]
            logging.error(error_msg)
            return None

    def _get_dns(self, db_name):
        """
        This is a private function which is used by :func:`delete` mehtod.
        Returns the *dns name* which is needed in order to set the *db_name* and *alias* to *NULL* (deletion). If the given *database name* which is passed as an argument does not exist then it returns None. 

        :param db_name: the new database name which is given in the url
        :type db_name: str
        :raises: IndexError - when the database name does not exist
        :rtype: str or None

        """
        composed_url = self.url + '?db_name=eq.' + db_name + '&select=dns_name'
        response = requests.get(composed_url)
        if response.ok:
            try:
                dns_name_dict = json.loads(response.text)[0]
                return dns_name_dict['dns_name']
            except IndexError:
                self.set_status(BAD_REQUEST)
                return None
        else:
            self.set_status(SERVICE_UNAVAILABLE) 
            return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import tornado.web
import json
import base64
import logging

from tornado.testing import AsyncHTTPTestCase
from tornado.testing import get_unused_port
from timeout_decorator import timeout

from apiato.api.api import *

class ClusterTest(AsyncHTTPTestCase):

    """Class to test instances endpoint"""
    authentication = "basic " + base64.b64encode(config.get('api','user') + ":" + config.get('api','pass'))

    def get_app(self):
        return tornado.web.Application(handlers, debug=True)

    @timeout(5)
    def test_get_volume_by_instance_name(self):

        """test getting a volume by name"""

        # Check the data for the given cluster
        print "test_volume_metadata"
        response = self.fetch("/api/v1/volume/apiato01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(len(data), 2)



    @timeout(5)
    def test_create_delete_volume(self):

        """test for create and delete a volume with the right data"""

        cluster = """{"instance_id": 3, "volume_type_id": "2", "server": "NAS-server", "mounting_path": "/MNT/data01", "attributes": [{ "test": "testvalue"}, {"test01" : "test01value" }], "file_mode": "0755", "owner": "TSM", "group": "ownergroup", "mount_options": "rw,bg,hard"}"""
        print "test_create_delete_volume"

        # Insert the volume
        response = self.fetch("/api/v1/volume/apiato03", method='POST', headers={'Authorization': self.authentication}, body=cluster)
        self.assertEquals(response.code, 201)

        # Check for the new volume
        response = self.fetch("/api/v1/volume/apiato03")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(len(data), 1)


        # Delete the created instance
        response = self.fetch("/api/v1/volume/" + str(data[0]["id"]) , method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)

        # Check again, the metadata should be empty
        response = self.fetch("/api/v1/volume/apiato03")
        self.assertEquals(response.code, 404)





# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import unittest
import logging

import base64
import tornado.web
import requests

from sys import stdout
from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from apiato.api.api import handlers
from apiato.config import config
logging.basicConfig(stream=stdout, level=logging.DEBUG)

class RundeckTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing Rundeck execution with nosetest"""
    
    authentication = "basic " + \
                     base64.b64encode(config.get('api', 'user') + \
                     ":" + config.get('api', 'pass'))

    def get_app(self):
        return tornado.web.Application(handlers)

    @patch('apiato.api.rundeck.requests.get')
    def test_get_success(self, mock_get):
        """test when get method is successful"""
        status_code_test = 200
        response_text = '[{"name":"apiato42","hostname":"apiato42.cern.ch","port":"5500","username":"apiato","db_type":"MYSQL","category":"TEST","tags":"MYSQL,TEST"}, \
        {"name":"apiato24","hostname":"apiato24.cern.ch","port":"6603","username":"apiato","db_type":"PG","category":"PROD","tags":"PG,PROD"}]'

        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          ok=True,
                                          status_code=status_code_test,
                                          text=response_text)

        response = self.fetch("/api/v1/rundeck/resources.xml")
        
        self.assertEquals(response.code, 200)

    @patch('apiato.api.rundeck.requests.get')
    def test_get_nosuccess(self, mock_get):
        """test when get method is not successful """
        status_code_test_error = 502

        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test_error)

        response = self.fetch("/api/v1/rundeck/resources.xml")
        self.assertEquals(response.code, 404)
    
    @timeout(10)
    @patch('apiato.api.rundeck.requests.get')
    @patch('apiato.api.rundeck.requests.post')
    def test_post_job_success(self, mock_post, mock_get):
        """test an execution of a registered job of an existing instance"""
        status_code_test = 200
        response_output_running = '{"execCompleted": false, "execState": "running"}'
        response_output_success = '{"execCompleted": true, "execState": "succeeded", "log": "[snapscript_24,snapscript_42]"}'
        response_run = '{"id":42}'

        mock_get.side_effect = [MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_output_running),
                                MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_output_success)]
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_run)

        response = self.fetch("/api/v1/rundeck/job/get-snapshots/instance42",
                             method="POST",
                             headers={'Authorization': self.authentication},
                             body='')
        self.assertEquals(response.code, 200)
        
    @patch('apiato.api.rundeck.requests.get')
    @patch('apiato.api.rundeck.requests.post')
    def test_post_job_nosuccess(self, mock_post, mock_get):
        """test when the job execution is not successful"""
        status_code_test = 200
        response_run = '{"id":42}'
        response_output = '{"execCompleted": true, "execState": "failed", "log": "[snapscript_24,snapscript_42]"}'

        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_output)
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                          ok=True,
                                          status_code=status_code_test,
                                          text=response_run)
        
        response = self.fetch("/api/v1/rundeck/job/get-snapshots/instance42",
                               method="POST",
                               headers={'Authorization': self.authentication},
                               body='')
        self.assertEquals(response.code, 502)
    
    @patch('apiato.api.rundeck.requests.get')
    @patch('apiato.api.rundeck.requests.post')
    def test_post_jobstatus_error(self, mock_post, mock_get):
        """test when the the get request of the job from rundeck status is not successful"""
        status_code_test = 200
        status_code_test_error = 500
        response_run = '{"id":42}'
        response_output = '{"execCompleted": true, "execState": "succeeded", "log": "[snapscript_24,snapscript_42]"}'

        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test_error,
                                          text=response_output)
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_run)
        
        response = self.fetch("/api/v1/rundeck/job/get-snapshots/instance42",
                               method="POST",
                               headers={'Authorization': self.authentication},
                               body='')
        self.assertEquals(response.code, status_code_test_error)

    @patch('apiato.api.rundeck.requests.get')
    @patch('apiato.api.rundeck.requests.post')
    def test_post_jobrun_error(self, mock_post, mock_get):
        """test when the the post request of the job to rundeck is not successful"""
        status_code_test = 200
        status_code_test_error = 400
        response_run = '{"id":42}'

        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          status_code=status_code_test)
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           ok=False,
                                           status_code=status_code_test_error,
                                           text=response_run)
        
        response = self.fetch("/api/v1/rundeck/job/get-snapshots/instance42",
                               method="POST",
                               headers={'Authorization': self.authentication},
                               body='')

        self.assertEquals(response.code, status_code_test_error)

    @patch('apiato.api.rundeck.requests.get')
    @patch('apiato.api.rundeck.requests.post')
    def test_post_job_timeout(self, mock_post, mock_get):
        """test an execution of a registered job giving timeout"""
        status_code_test = 200
        response_output_running = '{"execCompleted": false, "execState": "running"}'
        response_run = '{"id":42}'

        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_output_running)
        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_run)

        self.http_client.fetch(self.get_url("/api/v1/rundeck/job/get-snapshots/instance42"),
                     self.stop, method="POST", body='', headers={'Authorization': self.authentication})
        response = self.wait(timeout=25)
        self.assertEquals(response.code, 504)

    @patch('apiato.api.rundeck.requests.post')
    def test_post_job_bad_response(self, mock_post):
        """test when the the post request of the job to rundeck is not successful"""
        status_code_test = 200
        response_run = 'not_a_json_response'

        mock_post.return_value = MagicMock(spec=requests.models.Response,
                                           ok=True,
                                           status_code=status_code_test,
                                           text=response_run)
        
        response = self.fetch("/api/v1/rundeck/job/get-snapshots/instance42",
                               method="POST",
                               headers={'Authorization': self.authentication},
                               body='')

        self.assertEquals(response.code, 406)
    
    def test_post_job_noexisting(self):
        """test when the job does not exist"""
        response = self.fetch("/api/v1/rundeck/job/inexistent-job/dbod03",
                               method="POST",
                               headers={'Authorization': self.authentication},
                               body='')
        self.assertEquals(response.code, 400)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
REST API Server for the DB On Demand System
"""

import tornado.web
import logging
import requests
import json
import urllib

from apiato.api.base import *
from apiato.config import config

class Instance(tornado.web.RequestHandler):
    """
    This is the handler of **/instance/<database name>** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's tables/views that are used for this endpoint are 

        * *instance* - an example of this could look like this:

            +---+----------+---------++--------+-------+-----------+
            |id | username | db_name | db_type |version|    host   |
            +===+==========+=========+=========+=======+===========+
            |42 |dbod-usr42|dbod-db42| MYSQL   | 5.6.x |dbod-host42|
            +---+----------+---------+---------+-------+-----------+

        * *attribute*

            +--+------------+-------------------------+------------+
            |id| instance_id|           name          |  value     |
            +==+============+=========================+============+
            |24|     42     |port                     | 5432       |
            +--+------------+-------------------------+------------+
            |25|     42     |buffer_pool_size         | 1G         |
            +--+------------+-------------------------+------------+

        * *volume* - an example of this is like the following:

            +--+-------------+---------+-----------+-------------+
            |id| instance_id |file_mode|  server   |mounting_path|
            +==+=============+=========+===========+=============+
            |24|      42     |   0755  | NAS-server|  /MNT/data  |
        +--+-------------+---------+-----------+-------------+

        \* *(instance)id == (attribute/volume)instance_id*
        
        \*\* the id s are autoincremented (type serial)

      All of them provide the appropriate information for the creation/update/deletion of an instance.
  
    The request methods implemented for this endpoint are:

    * :func:`get`
    * :func:`post` - (creation)
    * :func:`put` - (update)
    * :func:`delete` - (deletion)

    .. note::

      You need to provide a <*username*> and a <*password*> or to provide
      manually the *Authorization* header in order to alter the database's
      content and specifically for :func:`post`, :func:`put` and :func:`delete`
      methods 

    """


    def get(self, name):
        """
        The *GET* method returns am *instance* given a *database name*.
        (No any special headers for this request)

        :param name: the database name which is given in the url
        :type name: str
        :rtype: json - the response of the request
        :raises: HTTPError - when the requested database name does not exist or if in case of an internal error 

        """
        response = requests.get(config.get('postgrest', 'instance_url') + "?name=eq." + name)
        if response.ok:
            data = response.json()
            if data:
                self.write({'response' : data})
                self.set_status(OK)
            else: 
                logging.error("Instance not found for name: " + name)
                raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Instance not found for name: " + name)
            raise tornado.web.HTTPError(NOT_FOUND)

    @http_basic_auth
    def post(self, id):
        """
        The *POST* method inserts a new instance into the database wih all the
        information that is needed for the creation of it.

        In the request body we specify all the information of the *instance*
        table along with the *attribute* and *volume* tables. All the 
        information is sent to a stored procedure in PostgreSQL which will
        insert the data in the related tables.
        
        .. note::
            
            * It's possible to insert more than one *hosts* or *volumes* in one instance.
            * The database names have to be unique
            * If any of the 3 insertions (in *instance*, *attribute*, *volume* table) is not successful then an *Exception* is raised and the private function :func:`__delete_instance__` is used in order to delete what may has been created.  
            * Also, the creation is not successful 

                * if the client is not authorized or
                * if there is any internal error
                * if the format of the request body is not right or if there is no *database name* field

        :param id: this argument is not used, however it has to exist for compatibility with the rest of endpoints
        :type id: str
        :raises: HTTPError - in case of an internal error
        :request body:  json

                       - for *instance*: json
                       - for *attribute*: json
                       - for *volume*: list of jsons

        """
        logging.debug(self.request.body)
        instance = {'in_json': json.loads(self.request.body)}
        
        # Insert the instance in database using PostgREST
        response = requests.post(config.get('postgrest', 'insert_instance_url'), json=instance, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Created instance " + instance["in_json"]["name"])
            logging.debug(response.text)
            self.write(response.text)
            self.set_status(CREATED)
        else:
            logging.error("Error creating the instance: " + response.text)
            raise tornado.web.HTTPError(response.status_code)
            
            
            
    @http_basic_auth
    def put(self, id):
        """
        The *PUT* method updates an instance into the database wih all the information that is needed.

        In the request body we specify all the information of the *instance*
        table along with the *attribute* and *volume* tables. 

        The procedure of this method is the following:

        * We extract and separate the information of each table. 
        * We get the *id* of the row from the given (unique) database from the url.
        * If it exists, we delete if any information with that *id* exists in the tables.
        * After that, we insert the information to the related table along with the instance *id*. 
        * In case of more than one attributes we insert each one separetely.  
        * Finally, we update the *instance* table's row (which include the given database id) with the new given information.

        :param id: the database id which is given in the url
        :type id: str
        :raises: HTTPError - when the *request body* format is not right or in case of internall error

        """
        logging.debug(self.request.body)
        instance = {'iid': id, 'in_json': json.loads(self.request.body)}
        
        # Update the instance in database using PostgREST
        response = requests.post(config.get('postgrest', 'update_instance_url'), json=instance, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Update instance: " + id)
            logging.debug(response.text)
            self.set_status(OK)
        else:
            logging.error("Error updating the instance: " + response.text)
            raise tornado.web.HTTPError(response.status_code)
            
            
            
    @http_basic_auth
    def delete(self, id):
        """
        The *DELETE* method deletes an instance by *database id*.
        
        In order to delete an instance we have to delete all the related information of the specified database id in *instance*, *attribute* and *volume* tables (:func:`__delete_instance__`).

        :param id: the database id which is given in the url
        :type id: str
        :raises: HTTPError - when the given database id cannot be found

        """
        logging.debug(self.request.body)
        if id.isdigit():
            instance = {'id': id}
        else:
            instance = {'instance_name': id}

        # Delete the instance in database using PostgREST
        response = requests.post(config.get('postgrest', 'delete_instance_url'), json=instance, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Delete instance %s" % (id) )
            logging.debug(response.text)
            self.set_status(NO_CONTENT)
        else:
            logging.error("Error deleting the instance: " + response.text)
            raise tornado.web.HTTPError(response.status_code)

class Instance_filter(tornado.web.RequestHandler):
    """
    This is the handler of **/api/v1/instance** endpoint.

    The request methods implemented for this endpoint are:

    * :func:`get`

    """

    get_instances_url = config.get('postgrest', 'get_instances_url')

    def get(self, *args):

        """
        The *GET* method returns a list of e_groups owning resources

        :rtype: json -- the response of the request
        :raises: HTTPError - if there is an internal error or if the response is empty
        """
        auth_header = self.request.headers.get('auth')
        logging.debug("Auth header : %s" % (auth_header))
        if auth_header is None:
            raise tornado.web.HTTPError(BAD_REQUEST, "No 'auth' header found.")

        try:
            auth = json.loads(auth_header)
        except:
            raise tornado.web.HTTPError(BAD_REQUEST, "Error parsing JSON 'auth' header.")

        logging.debug("RPC Url : %s" % (self.get_instances_url))
    
        response = requests.post(self.get_instances_url, json=auth,
                headers={'Prefer': 'return=representation'})

        if response.ok:
            self.write(response.text)
            self.set_status(OK)
        else:
            logging.error("Response: %s" % (response.text))
            raise tornado.web.HTTPError(response.status_code)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import tornado.web
import json
import urllib
import logging
import base64

from tornado.testing import AsyncHTTPTestCase
from tornado.testing import get_unused_port
from timeout_decorator import timeout

from apiato.api.api import *

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class InstanceTest(AsyncHTTPTestCase):
    """Class to test instances endpoint"""
    authentication = "basic " + base64.b64encode(config.get('api','user') + ":" + config.get('api','pass'))
    
    def get_app(self):
        return tornado.web.Application(handlers, debug=True)

    @timeout(5)
    def test_create_instance(self):
        """Creation of a new instance in a correct way"""
        instance = """{
        "owner": "testuser", "category": "TEST", "creation_date":"2016-07-20", 
        "version": "5.6.17", "type_id": 2, "host_id": 1, "name": "testdb", "state": "MAINTENANCE", "status": "ACTIVE",
        "volumes": [
            {"group": "ownergroup", "file_mode": "0755", "server": "NAS-server", "mount_options": "rw,bg,hard", "volume_type_id": 1,
            "owner": "TSM", "mounting_path": "/MNT/data1"}, 
            {"group": "ownergroup", "file_mode": "0755", "server": "NAS-server", "mount_options": "rw,bg,hard", "volume_type_id": 1,
            "owner": "TSM", "mounting_path": "/MNT/bin"}
        ],
        "attributes": [{
            "port": "5505",
            "notifications": "false"
        }]}"""
        
        # Create the instance
        response = self.fetch("/api/v1/instance/create", method='POST', headers={'Authorization': self.authentication}, body=instance)
        self.assertEquals(response.code, 201)
        created_id = json.loads(response.body)[0]["insert_instance"]
        logging.info("Created with ID: " + str(created_id))
        
        # Check the metadata for this new instance
        response = self.fetch("/api/v1/metadata/instance/testdb")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["db_name"], "testdb")
        self.assertEquals(len(data[0]["volumes"]), 2)
        self.assertEquals(len(data[0]["attributes"]), 2)
        self.assertEquals(data[0]["attributes"]["port"], "5505")  # Reminder: the port is saved as a String in DB
        self.assertEquals(data[0]["attributes"]["notifications"], "false")  # Reminder: the port is saved as a String in DB
        
        # Delete the created instance
        response = self.fetch("/api/v1/instance/" + str(created_id), method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)
        
        # Check again, the metadata should be empty
        response = self.fetch("/api/v1/metadata/instance/testdb")
        self.assertEquals(response.code, 404)
        
    @timeout(5)
    def test_create_basic_instance(self):
        """Creation of an instance with only basic (required) data"""
        instance = """{
        "owner": "testuser", "category": "TEST", "creation_date":"2016-07-20",
        "version": "5.6.17", "type_id": 2, "db_size": "100", "host_id": 1, "name": "testdb", "state": "MAINTENANCE", "status": "ACTIVE"
        }"""

        # Create the instance
        response = self.fetch("/api/v1/instance/testdb", method='POST', headers={'Authorization': self.authentication}, body=instance)
        self.assertEquals(response.code, 201)
        created_id = json.loads(response.body)[0]["insert_instance"]
        logging.info("Created with ID: " + str(created_id))

        # Check the metadata for this new instance
        response = self.fetch("/api/v1/metadata/instance/testdb")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["db_name"], "testdb")
        self.assertEquals(data[0]["hosts"][0], "host01")
        self.assertEquals(data[0]["username"], "testuser")
        self.assertEquals(data[0]["type"], "MYSQL")
        self.assertEquals(data[0]["version"], "5.6.17")
        self.assertTrue(not data[0]["volumes"])
        self.assertTrue(not data[0]["attributes"])

        # Delete the created instance
        response = self.fetch("/api/v1/instance/" + str(created_id), method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)

        # Check again, the metadata should be empty
        response = self.fetch("/api/v1/metadata/instance/testdb")
        self.assertEquals(response.code, 404)


    @timeout(5)
    def test_edit_instance_username(self):
        """Edit the owner correctly"""
        instance = """{"owner": "newuser"}"""
        restore = """{"owner": "user01"}"""

        # Edit the instance
        response = self.fetch("/api/v1/instance/1", method='PUT', headers={'Authorization': self.authentication}, body=instance)
        self.assertEquals(response.code, 200)

        # Check the metadata for this instance
        response = self.fetch("/api/v1/metadata/instance/apiato01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["username"], "newuser")

        # Restore the instance
        response = self.fetch("/api/v1/instance/1", method='PUT', headers={'Authorization': self.authentication}, body=restore)
        self.assertEquals(response.code, 200)

    @timeout(5)
    def test_edit_instance_dbname(self):
        """Edit the dbname correctly"""
        instance = """{"name": "newdb01"}"""
        restore = """{"name": "apiato01"}"""

        # Edit the instance
        response = self.fetch("/api/v1/instance/1", method='PUT', headers={'Authorization': self.authentication}, body=instance)
        self.assertEquals(response.code, 200)

        # Check the metadata for this instance
        response = self.fetch("/api/v1/metadata/instance/newdb01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["db_name"], "newdb01")

        # Restore the instance
        response = self.fetch("/api/v1/instance/1", method='PUT', headers={'Authorization': self.authentication}, body=restore)
        self.assertEquals(response.code, 200)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
Base module with common methods and classes used by another endpoints.
"""

import tornado.web
import base64
import functools
import logging
import requests
import json
import urllib

from apiato.config import config

# HTTP API status codes
OK = 200
CREATED = 201 # Request fulfilled resulting in creation of new resource
NO_CONTENT = 204 # Succesfull delete
NOT_FOUND = 404
UNAUTHORIZED = 401
BAD_REQUEST = 400
NOT_ACCEPTABLE = 406
BAD_GATEWAY = 502
GATEWAY_TIMEOUT = 504
SERVICE_UNAVAILABLE = 503

# Basic HTTP Authentication decorator
def http_basic_auth(fun):
    """Decorator for extracting HTTP basic authentication user/password pairs
    from the request headers and matching them to configurated credentials.
    It will generate an HTTP UNAUTHORIZED (Error code 401) if the request is 
    not using HTTP basic authentication.

    Example:
        @http_basic_auth
        def get(self, user, pwd)
    """
    @functools.wraps(fun)
    def wrapper(*args, **kwargs):
        """ Decorator wrapper """
        self = args[0]
        try:
            # Try
            auth = self.request.headers.get('Authorization')
            scheme, _, token = auth.partition(' ')
            if scheme.lower() == 'basic':
                # Decode user and password
                user, _, pwd = base64.decodestring(token).partition(':')
                if user == config.get('api','user') and pwd == config.get('api','pass'):
                    return fun(*args, **kwargs)
                else:
                    # Raise UNAUTHORIZED HTTP Error (401) 
                    logging.error("Unauthorized access from: %s",
                        self.request.headers)
                    raise tornado.web.HTTPError(UNAUTHORIZED)

            else:
                # We only support basic authentication
                logging.error("Authentication scheme not recognized")
                return "Authentication scheme not recognized"
        except AttributeError:
            # Raise UNAUTHORIZED HTTP Error (401) if the request is not
            # using autentication (auth will be None and partition() will fail
            logging.error("Unauthorized access from: %s",
                self.request.headers)
            raise tornado.web.HTTPError(UNAUTHORIZED)

    return wrapper
    
def get_instance_id_by_name(name):
    """Common function to get the ID of an instance by its name."""
    response = requests.get(config.get('postgrest', 'instance_url') + "?name=eq." + name)
    if response.ok:
        data = response.json()
        if data:
            return data[0]["id"]
    return None
    
class DocHandler(tornado.web.RequestHandler):
    """Shows the list of endpoints available in the API"""
    def get(self):
        logging.info("Generating API endpoints doc")
        response = """Please use :
            <p>http://hostname:port/api/v1/instance/NAME</p>
            <p>http://hostname:port/api/v1/instance/alias/NAME</p>
            <p>http://hostname:port/api/v1/host/aliases/HOSTNAME</p>
            <p>http://hostname:port/api/v1/metadata/instance/NAME</p>
            <p>http://hostname:port/api/v1/metadata/host/HOSTNAME</p>
            <p>http://hostname:port/api/v1/rundeck/resources.xml</p>
            <p>http://hostname:port/api/v1/rundeck/job/JOB/NODE</p>"""
        self.set_header("Content-Type", 'text/html')
        self.write(response)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Testing functional alias endpoint"""

import json
import unittest
import base64
import requests
import tornado.web

from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from apiato.api.api import handlers
from apiato.config import config

class FunctionalAliasTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing functional alias with nosetest"""
    #headers = {'Content-Type': 'application/x-www-form-urlencoded', 
    #           'Prefer': 'return=representation',
    #           'Accept': 'text/json'}
    db_name_test = "apiato42"
    alias_test = "apiato-apiato-42.cern.ch"
    authentication = "basic " + \
                     base64.b64encode(config.get('api', 'user') + \
                     ":" + config.get('api', 'pass'))
    
    def get_app(self):
        return tornado.web.Application(handlers)

    @timeout(5)
    def test_get_single_alias_by_name(self):
        """test for getting the right data"""
        print "test_get_single_alias_by_name"
        db_name = 'apiato01'
        response = self.fetch("/api/v1/instance/alias/%s" %(db_name))
        data = json.loads(response.body)["response"]
        self.assertEquals(response.code, 200)
        #self.assertEquals(json.loads(response.headers)['Content-Type'], 'application/json')
        self.assertEquals(len(data), 1)
        self.assertEquals(data[0]["alias"], "apiato-apiato-01.cern.ch")
        self.assertTrue(data[0]["dns_name"] != None)
        self.assertEquals(response.headers['Content-Type'], 'application/json; charset=UTF-8')

    @timeout(5)
    def test_get_invalid_dbname(self):
        """test when the db_name does not exist"""
        print "test_get_invalid_dbname"
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test))
        data = response.body
        self.assertEquals(response.code, 404)
        self.assertEquals(len(data), 69)
        self.assertEquals(response.headers['Content-Type'], 'text/html; charset=UTF-8')

    @timeout(5)
    @patch('apiato.api.functionalalias.requests.get')
    def test_get_bad_response(self, mock_get):
        """test when the get response code is not 200. Server/api error"""
        print "test_get_bad_response"
        status_code_test = 503
        mock_get.return_value = MagicMock(spec=requests.models.Response, 
                                          ok=False,
                                          status_code=status_code_test)
        db_name = 'apiato01'
        response = self.fetch("/api/v1/instance/alias/%s" %(db_name))
        self.assertEquals(response.code, status_code_test)
        self.assertEquals(response.headers['Content-Type'], 'text/html; charset=UTF-8')

    @timeout(5)
    def test_novalid_db(self):
        """test when the given db does not exist"""
        response = self.fetch("/api/v1/instance/alias/some_db")
        self.assertEquals(response.code, 404)


    @timeout(5)
    def test_post_valid_request(self):
        """test when the arguments are valid and dns_name is available"""
        print "test_post_valid_request"
        body = 'alias=' + self.alias_test
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)  
        self.assertEquals(response.code, 201)
        # confirm you get back the right data
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test))
        data = json.loads(response.body)["response"]
        self.assertEquals(len(data), 1)
        self.assertEquals(data[0]["alias"], self.alias_test)
        self.assertTrue(data[0]["dns_name"] != None)
        # delete what has been created
        self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                   headers={'Authorization': self.authentication},
                   method="DELETE")

    @timeout(5)
    def test_post_duplicate(self):
        """test when there is a request to insert a db_name which already exists"""
        print "test_post_duplicate"
        body = 'alias=apiato-apiato-01.cern.ch'
        response = self.fetch("/api/v1/instance/alias/apiato01", 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)
        self.assertEquals(response.code, 409)    

    @timeout(5)
    def test_post_no_dns(self):
        """test when there are no any dns available"""
        print "test_post_no_dns"
        body = 'alias=' + self.alias_test
        self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                   method="POST", 
                   headers={'Authorization': self.authentication},
                   body=body)
        body = 'alias=' + 'apiato-apiato-24.cern.ch'
        response = self.fetch("/api/v1/instance/alias/apiato24", 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)
        self.assertEquals(response.code, 503)
        # delete what has been created
        self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                   headers={'Authorization': self.authentication},
                   method="DELETE")


    @timeout(5)
    def test_post_no_valid_argument(self):
        """test if the provided argument is not valid"""
        print "test_post_no_valid_argument"
        body = 'something=%s' + self.alias_test
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)
        self.assertEquals(response.code, 400)

    @timeout(5)
    def test_post_bad_argument(self):
        """test if the provided value of the argument is not valid"""
        print "test_post_bad_argument"
        body = 'alias ' + self.alias_test
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)
        self.assertEquals(response.code, 400)
    
    @timeout(5)
    @patch('apiato.api.functionalalias.requests.get')
    def test_post_nextdns_failure(self, mock_get):
        """test when there is a server error when getting an available dns_name"""
        print "test_post_nextdns_failure"
        status_code_test = 503
        mock_get.return_value = MagicMock(spec=requests.models.Response, 
                                          ok=False,
                                          status_code=status_code_test)
        
        body = 'alias=%s' + self.alias_test
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)  
        # tornado will still raise a Bad Request -- to be improved 
        self.assertEquals(response.code, 503)
        # delete what has maybe been created
        self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                   headers={'Authorization': self.authentication},
                   method="DELETE")   

    @timeout(5)
    def test_delete_valid_request(self):
        """test when there is a valid request to delete a previous inserted db_name"""
        print "test_delete_valid_request"
        # create entity to be deleted
        body = 'alias=' + self.alias_test
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              method="POST", 
                              headers={'Authorization': self.authentication},
                              body=body)
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              headers={'Authorization': self.authentication},
                              method="DELETE")
        self.assertEquals(response.code, 204)

    @timeout(5)
    def test_delete_invalid_dbname(self):
        """test when the given db_name to be deleted does not exist"""
        print "test_delete_invalid_dbname"
        response = self.fetch("/api/v1/instance/alias/%s" %(self.db_name_test), 
                              headers={'Authorization': self.authentication},
                              method="DELETE")
        self.assertEquals(response.code, 400)

    @timeout(5)
    @patch('apiato.api.functionalalias.requests.get')
    def test_delete_getdns_failure(self, mock_get):
        """test an unsuccessful get of the dns_name"""
        print "test_delete_getdns_failure"
        status_code_test = 503
        mock_get.return_value = MagicMock(spec=requests.models.Response,
                                          ok=False,
                                          status_code=status_code_test)
        response = self.fetch("/api/v1/instance/alias/%s" %('apiato01'),
                              headers={'Authorization': self.authentication},
                              method="DELETE")
        # tornado will still give Bad Request error message -- to be improved
        self.assertEquals(response.code, 503)

    @timeout(5)
    @patch('apiato.api.functionalalias.requests.patch')
    def test_delete_nosuccess(self, mock_patch):
        """test an unsuccessful deletion"""
        print "test_delete_nosuccess_delete"
        status_code_test = 503
        mock_patch.return_value = MagicMock(spec=requests.models.Response,
                                            ok=False,
                                            status_code=status_code_test) 
        response = self.fetch("/api/v1/instance/alias/%s" %('apiato01'),
                              headers={'Authorization': self.authentication},
                              method="DELETE")
        self.assertEquals(response.code, status_code_test)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
FIM module, which has the endpoint to read FIM's data
"""

import tornado.web
import logging
import requests
import json

from apiato.api.base import *
from apiato.config import config

class Fim(tornado.web.RequestHandler):
    """
    This is the handler of **/fim/<name>** endpoint.

    This endpoint takes 1 arguments:

    * *<name>* - the name of a *database* 

    Things that are given for the development of this endpoint:
    
    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's table/view that is used for this endpoint is called *db_on_demand* and provides information about the metadata of a database instance(s).

    The request method implemented for this endpoint is just the :func:`get`.

    """
    def get(self, name):
        """Returns the FIM's data for an instance
        (No any special headers for this request)

        :param name: the database name which is given in the url
        :type name: str
        :rtype: json - the response of the request, which will include all the information for the given database

        :raises: HTTPError - whene th given database name does not exist or in case of an internal error

        """
        
        response = requests.get(config.get('postgrest', 'fim_url') + '?instance_name=eq.' + name, verify=False)
        if response.ok:
            data = response.json()
            if data:
                logging.debug("data: " + json.dumps(data))
                self.write({'data' : data})
                self.set_status(OK)
            else:
                logging.error("Instance not found in FIM: " + name)
                raise tornado.web.HTTPError(NOT_FOUND)
        else:
            logging.error("Error fetching instance information: " + response.text)
            raise tornado.web.HTTPError(response.status_code)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2016, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import logging
import unittest
import sys
import os.path

from apiato.config import load

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class ConfigTest(unittest.TestCase):
    """Test configuration loading"""

    def test_config_file_not_found(self):
        with self.assertRaises(SystemExit) as cm:
            config = load('/path/to/unexisting/file')
        self.assertEqual(cm.exception.code, 1)

    @unittest.skipUnless(os.path.isfile('/tmp/api_missing_section.cfg'),'Skipping as no missing section file was found')
    def test_config_missing_section(self):
        with self.assertRaises(SystemExit) as cm:
            config = load('/tmp/api_missing_section.cfg')
        self.assertEqual(cm.exception.code, 2)
    
    @unittest.skipUnless(os.path.isfile('/tmp/api_missing_option.cfg'),'Skipping as no missing section file was found')
    def test_config_missing_option(self):
        with self.assertRaises(SystemExit) as cm:
            config = load('/tmp/api_missing_option.cfg')
        self.assertEqual(cm.exception.code, 2)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
DB On Demand REST API server setup file
"""

from setuptools import setup, find_packages

setup(name='apiato',
      version='0.9.4',
      description='CERN Apiato REST API',
      author='CERN',
      author_email='icot@cern.ch',
      license='GPLv3',
      maintainer='Ignacio Coterillo',
      maintainer_email='icot@cern.ch',
      url='https://github.com/cerndb/dbod-api',
      packages=find_packages(),
      scripts=['bin/apiato'],
      test_suite="",
      install_requires=[
          'ConfigParser',
          'tornado',
          'nose',
          'mock',
          'requests',
          'timeout_decorator',
          ],
     )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
REST API Server for the DB On Demand System
"""

import tornado.web
import logging
import requests
import json
import urllib

from apiato.api.base import *
from apiato.config import config

class Job(tornado.web.RequestHandler):

    @http_basic_auth
    def get(self, **args):
        job_id = args.get('id')
        db_name = args.get('db_name')

        # Get instance information
        instance_id = get_instance_id_by_name(db_name)
        
        if not instance_id:
            logging.error("Instance not found for name: " + db_name)
            raise tornado.web.HTTPError(NOT_FOUND)

        if job_id:
            # Get an specific job
            response = requests.get(config.get('postgrest', 'job_log_url') + "?id=eq." + job_id)
            if response.ok:
                data = response.json()
                if data:
                    logging.debug("Received data: " + str(data))
                    if int(data[0]["instance_id"]) == instance_id:
                        self.write({'response' : data[0]})
                        self.set_status(OK)
                    else:
                        logging.error("No rights to access job id: " + job_id + " from instance: " + db_name)
                        raise tornado.web.HTTPError(UNAUTHORIZED)
                else: 
                    logging.error("Job id not found: " + job_id)
                    raise tornado.web.HTTPError(NOT_FOUND)
        else:
            # Get all jobs for this instance starting from the most recent one
            response = requests.get(config.get('postgrest', 'job_url') + "?instance_id=eq." + str(instance_id))
            if response.ok:
                data = response.json()
                logging.debug(data)
                if data:
                    self.write({'response' : data})
                    self.set_status(OK)
                else:
                    self.write({'response' : []})
                    self.set_status(OK)
            else:
                logging.error("Error getting jobs for instance: " + db_name)
                raise tornado.web.HTTPError(BAD_REQUEST)

class Job_filter(tornado.web.RequestHandler):
    """
    This is the handler of **/api/v1/job** endpoint.

    The request methods implemented for this endpoint are:

    * :func:`get`

    """

    get_jobs_url = config.get('postgrest', 'get_jobs_url')

    def get(self, *args):
        """
        The *GET* method returns a list of e_groups owning resources

        :rtype: json -- the response of the request
        :raises: HTTPError - if there is an internal error or if the response is empty
        """
        auth_header = self.request.headers.get('auth')
        logging.debug("Auth header : %s" % (auth_header))
        if auth_header is None:
            raise tornado.web.HTTPError(BAD_REQUEST, "No 'auth' header found.")
        
        try:
            auth = json.loads(auth_header)
        except:
            raise tornado.web.HTTPError(BAD_REQUEST, "Error parsing JSON 'auth' header.")

        logging.debug("RPC Url : %s" % (self.get_jobs_url))

        response = requests.post(self.get_jobs_url, json=auth, 
                headers={'Prefer': 'return=representation'})

        if response.ok:
            self.write(response.text)
            self.set_status(OK)
        else:
            logging.error("Response: %s" % (response.text))
            raise tornado.web.HTTPError(response.status_code)




# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import unittest
from types import *
import json
import logging
import sys
import requests

import base64
import tornado.web

from mock import patch
from mock import MagicMock
from tornado.testing import AsyncHTTPTestCase
from timeout_decorator import timeout

from apiato.api.api import handlers
from apiato.config import config
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class JobTest(AsyncHTTPTestCase, unittest.TestCase):
    """Class for testing Job endpoint with nosetest"""
    
    authentication = "basic " + \
                     base64.b64encode(config.get('api', 'user') + \
                     ":" + config.get('api', 'pass'))
    
    def get_app(self):
        return tornado.web.Application(handlers)
        
    @timeout(5)
    def create_instance(self):
        """Create a new empty instance to run the tests to it"""
        response = self.fetch("/api/v1/instance/testjobs", method='DELETE', headers={'Authorization': self.authentication})
        
        instance = """{
        "owner": "testuser", "category": "TEST", "creation_date":"2016-07-20", 
        "version": "5.6.17", "type_id": 2, "host_id": 1, "name": "testjobs", "state": "MAINTENANCE", "status": "ACTIVE",
        "volumes": [
            {"group": "ownergroup", "file_mode": "0755", "server": "NAS-server", "mount_options": "rw,bg,hard", "volume_type_id": 1,
            "owner": "TSM", "mounting_path": "/MNT/data1"}, 
            {"group": "ownergroup", "file_mode": "0755", "server": "NAS-server", "mount_options": "rw,bg,hard", "volume_type_id": 1,
            "owner": "TSM", "mounting_path": "/MNT/bin"}
        ],
        "attributes": [{
            "port": "5505",
            "notifications": "false"
        }]}"""
        
        # Create the instance
        response = self.fetch("/api/v1/instance/testjobs", method='POST', headers={'Authorization': self.authentication}, body=instance)
        
    @timeout(5)
    def delete_instance(self):
        """Destroy the created test instance"""
        response = self.fetch("/api/v1/instance/testjobs", method='DELETE', headers={'Authorization': self.authentication})
        
    @timeout(5)
    def test_get_jobs_empty(self):
        """Get the jobs of the instance when there are no jobs"""
        self.create_instance()
        
        response = self.fetch("/api/v1/instance/testjobs/job", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body).get('response')
        self.assertEquals(len(response), 0)
        
        self.delete_instance()
        
    @timeout(5)
    def test_get_jobs_one(self):
        """Get the jobs of an instance having only one job"""
        response = self.fetch("/api/v1/instance/apiato02/job", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body).get('response')
        self.assertEquals(len(response), 1)
        self.assertEquals(response[0]["username"], "user02")
        self.assertEquals(response[0]["db_name"], "apiato02")
        
    @timeout(5)
    def test_get_jobs_many(self):
        """Get the jobs of an instance having many jobs"""
        response = self.fetch("/api/v1/instance/apiato01/job", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 200)
        
        # Check the port value is correct
        response = json.loads(response.body).get('response')
        self.assertEquals(len(response), 5)
        self.assertEquals(response[0]["db_name"], "apiato01")
        self.assertEquals(response[1]["db_name"], "apiato01")
        self.assertEquals(response[2]["db_name"], "apiato01")
        self.assertEquals(response[3]["db_name"], "apiato01")
        self.assertEquals(response[4]["db_name"], "apiato01")
        
    @timeout(5)
    def test_get_job(self):
        """Get a job correctly"""
        response = self.fetch("/api/v1/instance/apiato01/job/1", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 200)
        
        response = json.loads(response.body).get('response')
        self.assertIsNotNone(response["log"])
        
    @timeout(5)
    def test_get_job_different_instance(self):
        """Get a job from different instance"""
        response = self.fetch("/api/v1/instance/apiato01/job/6", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 401)
        
    @timeout(5)
    def test_get_job_no_exists(self):
        """Get an inexistent job"""
        response = self.fetch("/api/v1/instance/apiato01/job/60000", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 404)
        
    @timeout(5)
    def test_get_job_instance_no_exists(self):
        """Get a job from inexistent instance"""
        response = self.fetch("/api/v1/instance/noexists/job/6", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 404)
        
    @timeout(5)
    def test_get_jobs_instance_no_exists(self):
        """Get jobs from inexistent instance"""
        response = self.fetch("/api/v1/instance/noexists/job", headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 404)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

"""
REST API Server for the DB On Demand System
"""

import tornado.web
import logging
import requests
import json
import urllib

from apiato.api.base import *
from apiato.config import config

class Attribute(tornado.web.RequestHandler):
    """
    This is the handler of **/<class>/<name>/attribute/<attribute_name>** endpoint.

    Things that are given for the development of this endpoint:

    * We request indirectly a `Postgres <https://www.postgresql.org/>`_ database through `PostgREST <http://postgrest.com/>`_ which returns a response in JSON format
    * The database's tables/views that are used for this endpoint are 

        * *attribute*

            +--+------------+-------------------------+------------+
            |id| instance_id|           name          |  value     |
            +==+============+=========================+============+
            |24|     42     |port                     | 5432       |
            +--+------------+-------------------------+------------+
            |25|     42     |buffer_pool_size         | 1G         |
            +--+------------+-------------------------+------------+

        \* *(instance)id == (attribute/volume)instance_id*
        
        \*\* the id s are autoincremented (type serial)

      All of them provide the appropriate information for the creation/update/deletion of an instance.
  
    The request methods implemented for this endpoint are:

    * :func:`get`
    * :func:`post` - (creation/addition of attributes)
    * :func:`put` - (update of existing attributes)
    * :func:`delete` - (deletion of attributes)
    
    .. note::

      You need to provide a <*username*> and a <*password*> or to provide
      manually the *Authorization* header in order to alter the database's
      content and specifically for :func:`post`, :func:`put` and :func:`delete`
      methods 

    """

    def get(self, **args):
        """
        The *GET* method returns an attribute value from a certain *instance*.
        The parameter <attribute name> is optional. If it's not set the method
        will return all the attributes referred to the instance.  (No any
        special headers for this request)
        
        :param inst_name: the database name which is given in the url
        :type inst_name: str
        :param attr_name: the attribute name to return
        :type attr_name: str
        :rtype: str/json - the response of the request. A string for a single attribute
          and a json encoded object (a dictionary of attribute_name:value structure.
        :raises: HTTPError - when the requested database name does not exist or
            if in case of an internal error 

        """
        instance_name = args.get('entity')
        attribute_name = args.get('attribute_name')
        class_n = args.get('class')
        url = None
        input = None
        pg_method_name = None
        if attribute_name:
            url = config.get('postgrest', class_n + '_attribute_url')
            input = {
                    'instance_name': instance_name, 
                    'attribute_name': attribute_name
                    }
            pg_method_name = 'get_instance_attribute'
        else:
            url = config.get('postgrest', class_n + '_attributes_url')
            input = { 'instance_name': instance_name }
            pg_method_name = 'get_instance_attributes'
        
        logging.debug("Requesting %s (%s)" % (url, input))
        response = requests.post(url, json=input, headers ={'Prefer': 'return=representation'})

        if response.ok:
            data = response.json()
            logging.debug("Response data: %s" % (data))
            resp = data[0].get(pg_method_name)
            if resp:
                logging.debug("Returning data: %s" % (resp))
                if attribute_name:
                    resbody = json.dumps({'value': resp.strip('\"')})
                    self.write(resbody)
                else:
                    self.write(resp)
                self.set_status(OK)
            else:
                self.set_status(NOT_FOUND)
        else:
            logging.error("Attributes not found for " + class_n + ": " + instance_name)
            raise tornado.web.HTTPError(NOT_FOUND)

    @http_basic_auth
    def post(self, **args):
        """
        The *POST* method inserts a new attribute into the database for the specified instance.

        In the request body we specify all the information of the *attribute*
        table.
        
        .. note::
            
            * It's possible to insert more than one *attributes* in the same request. 
            * The attribute names are unique for each instance.
            * The creation is not successful 

                * if the client is not authorized or
                * if there is any internal error
                * if the format of the request body is not right or if there is no *database name* field

        :param instance_name: the name of the entity to insert the attributes to
        :type name: str
        :raises: HTTPError - in case of an internal error
        :request body:  json

        """
        instance_name = args.get('entity')
        class_n = args.get('class')
        
        logging.debug(self.request.body)
        in_json = json.loads(self.request.body)
        # Raise error if the attribute is Null
        for k, v in in_json.items():
            if v == None:
                raise tornado.web.HTTPError(BAD_REQUEST)

        attribute = {'instance_name': instance_name, 'in_json': in_json}

        # Insert the instance in database using PostREST
        response = requests.post(config.get('postgrest', 'insert_' + class_n + '_attribute_url'), json=attribute, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Created attribute " + json.dumps(in_json) + " for " + class_n + " " + instance_name)
            logging.debug(response.text)
            self.set_status(CREATED)
        else:
            code = None
            if response.status_code == 400:
                # Instance doesn't exist, id is null and the insert fails with this code
                code = NOT_FOUND
            else:
                code = response.status_code
            logging.error("Error creating the attribute for " + class_n + " " + instance_name + ": " + response.text)
            raise tornado.web.HTTPError(response.status_code)
            
    @http_basic_auth
    def put(self, **args):
        """
        The *PUT* method updates an attribute into the database wih all the
        information that is needed.  The name of the instance and the attribute
        are set in the URL. The new value of the attribute must be sent in the
        *request body*.

        :param instance_name: the database name which is given in the url
        :type instance: str
        :param attribute_name: the attribute name which is given in the url
        :type attribute: str
        :raises: HTTPError - when the *request body* format is not right or in case of internall error

        """
        
        instance_name = args.get('entity')
        attribute_name = args.get('attribute_name')
        class_n = args.get('class')
        
        logging.debug(self.request.body)
        in_json = {
                'instance_name': instance_name, 
                'attribute_name': attribute_name,
                'attribute_value': self.request.body
                }

        logging.debug("BODY: %s" % (self.request.body))

        response = requests.post(config.get('postgrest', 'update_' + class_n + '_attribute_url'), json=in_json, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Updated " + class_n + " attribute: " + attribute_name)
            self.set_status(OK)
        else:
            logging.error("Error updating the " + class_n + " " + instance_name + " attribute: " + response.text)
            raise tornado.web.HTTPError(response.status_code)

    @http_basic_auth
    def delete(self, **args):
        """
        The *DELETE* method deletes an attribute by *instance name* and *attribute name*.
        
        :param instance_name: the database name which is given in the url
        :type instance: str
        :param attribute_name: the attribute name which is given in the url
        :type attribute: str
        :raises: HTTPError - when the given database name cannot be found

        """
        instance_name = args.get('entity')
        attribute_n = args.get('attribute_name')
        class_n = args.get('class')
        
        json = {'instance_name': instance_name, 'attribute_name': attribute_n}
        
        # Delete the attribute in database using PostREST
        response = requests.post(config.get('postgrest', 'delete_' + class_n + '_attribute_url'), json=json, headers={'Prefer': 'return=representation'})
        if response.ok:
            logging.info("Delete " + class_n + " attribute: " + attribute_n)
            logging.debug(response.text)
            self.set_status(NO_CONTENT)
        else:
            logging.error("Error deleting the " + class_n + " " + instance_name + " attribute: " + response.text)
            raise tornado.web.HTTPError(response.status_code)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2015, CERN
# This software is distributed under the terms of the GNU General Public
# Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".
# In applying this license, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as Intergovernmental Organization
# or submit itself to any jurisdiction.

import tornado.web
import json
import base64
import logging

from tornado.testing import AsyncHTTPTestCase
from tornado.testing import get_unused_port
from timeout_decorator import timeout

from apiato.api.api import *

class ClusterTest(AsyncHTTPTestCase):

    """Class to test instances endpoint"""
    authentication = "basic " + base64.b64encode(config.get('api','user') + ":" + config.get('api','pass'))

    def get_app(self):
        return tornado.web.Application(handlers, debug=True)

    @timeout(5)
    def test_get_cluster_by_name(self):

        """test getting a cluster by name"""

        # Check the data for the given cluster
        print "test_cluster_metadata"
        response = self.fetch("/api/v1/cluster/cluster01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["name"], "cluster01")
        self.assertEquals(len(data[0]["instances"]), 2)
        self.assertEquals(len(data[0]["attributes"]), 2)
        self.assertEquals(data[0]["attributes"]["user"], "zookeeper")



    @timeout(5)
    def test_create_delete_cluster(self):

        """test for create and delete a cluster with the right data"""

        cluster = """{"owner": "testuser", "category": "TEST", "creation_date": "2016-11-20", "e_group": "testgroupZ", "version": "3.9", "type_id": 1, "name": "testcluster", "state": "RUNNING", "status": "ACTIVE", "attributes": [{"testp01" : "testvalue" },{"testp02" :"testvalue"}]}"""
        print "test_create_delete_cluster"

        # Create the instance
        response = self.fetch("/api/v1/cluster/create", method='POST', headers={'Authorization': self.authentication}, body=cluster)
        self.assertEquals(response.code, 201)

        # Check for this new cluster
        response = self.fetch("/api/v1/cluster/testcluster")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["name"], "testcluster")
        self.assertEquals(len(data[0]["attributes"]), 2)
        self.assertEquals(data[0]["attributes"]["testp01"], "testvalue")  # Reminder: the port is saved as a String in DB

        # Delete the created instance
        response = self.fetch("/api/v1/cluster/" + str(data[0]["id"]) , method='DELETE', headers={'Authorization': self.authentication})
        self.assertEquals(response.code, 204)

        # Check again, the metadata should be empty
        response = self.fetch("/api/v1/cluster/testcluster")
        self.assertEquals(response.code, 404)


    def test_update_cluster(self):
        """test for update a cluster"""
        current_cluster = """{ "e_group" : "testgroupZ"}"""
        new_cluster = """{ "e_group" : "testgroupX"}"""

        response = self.fetch("/api/v1/cluster/1" , method='PUT', headers={'Authorization': self.authentication}, body=new_cluster)
        self.assertEquals(response.code, 200)

        # Check the cluster update
        response = self.fetch("/api/v1/cluster/cluster01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["e_group"], "testgroupX")

        # Restore the instance
        response = self.fetch("/api/v1/cluster/1", method='PUT', headers={'Authorization': self.authentication}, body=current_cluster)
        self.assertEquals(response.code, 200)

        # Check the cluster update (restore)
        response = self.fetch("/api/v1/cluster/cluster01")
        self.assertEquals(response.code, 200)
        data = json.loads(response.body)["response"]
        self.assertEquals(data[0]["e_group"], "testgroupZ")


    @timeout(5)
    def test_get_invalid_cluster(self):
        print "test_get_invalid_cluster"
        response = self.fetch("/api/v1/cluster/invalid")
        self.assertEquals(response.code, 404)
